################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
SVGTree.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, getopt, time, optparse, math, tempfile, bisect

""" program $Id: SVGTree.py 2784 2009-09-10 11:41:14Z andreas $
class to plot a phylogenetic tree as an SVG file.
"""
import Experiment
import IOTools
import SVGdraw 
import TreeTools
import Tree

# some definitions for the layout of the picture
BLACK = (0,0,0)
WHITE = (255,255,255)
RED   = (255,0,0)
GREEN = (0,255,0)
BLUE  = (0,0,255)
YELLOW = (255,255,0)
CYAN   = (0,255,255)
PURPLE = (255,0,255)
GREY = (128,128,128)
ORANGE = (255,165,0)
PINK = (255,192,203)
PLUM= (221,160,221)
OLIVE = (128,128,0)
BROWN = (165,42,42)

COLOURS= (BLACK, RED, GREEN, BLUE, YELLOW, CYAN, PURPLE, GREY, ORANGE, PINK,
          PLUM, OLIVE, BROWN)

MAX_GREY = 240
GREY_COLORS = map( lambda x: (x,x,x), range(0,MAX_GREY))

###################################################################
class BorderDecorator:
    """class for decorating the border in a tree plot."""

    mFontSize = 36
    mFont = "Verdana"
    
    def __init__(self, tree):
        self.mTree = tree

class BorderDecoratorLocations(BorderDecorator):

    mTickWidth = 10
    mDefaultColour = BLACK
    
    def __init__(self, tree, map_id2location,
                 extract_species = None, map_species2colour = None,
                 max_separation = 0,
                 print_location = False,
                 *args, **kwargs):

        """
        provide extract_species and map_species2colour to colour
        by species.
        
        max_separation: maximum separation between syntenic locations.
        set 0, if any distance is ok and only synteny is enough.
        """
        BorderDecorator.__init__(self, tree, *args, **kwargs)

        self.mMaxSeparation = max_separation        
        self.mMapId2Location = map_id2location
        self.mExtractSpecies = extract_species
        self.mMapSpecies2Colour = {}

        if map_species2colour:
            for s, c in map_species2colour.items():
                self.mMapSpecies2Colour[s] = tuple(map(int, c.split(",")))
        
        self.mPrintLocation = print_location
        
    def getWidth( self ):

        t = self.mTree.get_terminals()

        ## check if we have terminals which need to be connected
        count = 0
        for i in range(len(t)-1):
            node_id1 = t[i]
            taxon1 = self.mTree.node(node_id1).data.taxon

            for j in range(i+1, len(t)):
                node_id2 = t[j]

                taxon2 = self.mTree.node(node_id2).data.taxon                
                
                l1 = self.mMapId2Location[taxon1]
                l2 = self.mMapId2Location[taxon2]                
                if l1.contig != l2.contig:
                    continue

                if self.mMaxSeparation:
                    s = min( abs(l1.mFrom - l2.mTo), abs(l1.mTo - l2.mFrom))
                    if s >= self.mMaxSeparation: continue

                count += 1

        if count == 0:
            return 0
        
        ## rule of thumb
        if self.mPrintLocation:
            return 400
        else:
            return 100
    
    def getElements(self, x, y, map_node2height ):

        t = self.mTree.get_terminals()
        
        elements = []

        ## print locations
        if self.mPrintLocation:
            for i in range(len(t)):
                node_id1 = t[i]
                taxon1 = self.mTree.node(node_id1).data.taxon
                y1 = map_node2height[node_id1] + y

                elements.append( SVGdraw.text( x, y1,
                                               str(self.mMapId2Location[taxon1]),
                                               self.mFontSize,
                                               self.mFont,
                                               stroke = "rgb(%i,%i,%i)" % BLACK,
                                               text_anchor = "left" ))

        
        ## print connectors
        for i in range(len(t)-1):
            node_id1 = t[i]
            taxon1 = self.mTree.node(node_id1).data.taxon
            y1 = map_node2height[node_id1] + y

            for j in range(i+1, len(t)):
                node_id2 = t[j]

                taxon2 = self.mTree.node(node_id2).data.taxon                

                if self.mExtractSpecies:
                    species1 = self.mExtractSpecies(taxon1)
                    species2 = self.mExtractSpecies(taxon2)

                    if species1 != species2: continue

                    if species1 not in self.mMapSpecies2Colour:
                        self.mMapSpecies2Colour[species1] = COLOURS[len(self.mMapSpecies2Colour) % len(COLOURS) ]

                    colour = self.mMapSpecies2Colour[species1]
                    
                else:
                    colour = self.mDefaultColour
                    
                l1 = self.mMapId2Location[taxon1]
                l2 = self.mMapId2Location[taxon2]                
                if l1.contig != l2.contig:
                    continue

                if self.mMaxSeparation:
                    s = min( abs(l1.mFrom - l2.mTo), abs(l1.mTo - l2.mFrom))
                    if s >= self.mMaxSeparation: continue
                    
                y2 = map_node2height[node_id2] + y

                distance = y2 - y1

                d = SVGdraw.pathdata( x, y1 )

                d.line( x + self.mTickWidth, y1 )
                d.ellarc( distance, distance, 0, 0, 1, x + self.mTickWidth, y2 )
                d.line( x, y2 )                

                
                e = SVGdraw.path( d,
                                  fill = "none",
                                  stroke = "rgb(%i,%i,%i)" % colour,
                                  stroke_width = 1 )

                elements.append( e )

        return elements

class BorderDecoratorClusters(BorderDecorator):

    mBoxWidth = 10
    mDefaultColour = BLACK
    
    def __init__(self, tree, map_id2cluster,
                 map_cluster2colour = None,
                 *args, **kwargs):

        """
        plot a box according to cluster.
        
        """
        BorderDecorator.__init__(self, tree, *args, **kwargs)

        self.mMapId2Cluster = map_id2cluster
        self.mMapCluster2Colour = {}

        if map_cluster2colour:
            for s, c in map_cluster2colour.items():
                self.mMapCluster2Colour[s] = tuple(map(int, c.split(",")))
        
    def getWidth( self ):
        return self.mBoxWidth
    
    def getElements(self, x, y, map_node2height ):

        elements = []
        for n in self.mTree.get_terminals():

            t = self.mTree.node(n).data.taxon

            if t not in self.mMapId2Cluster:
                continue
            cluster = self.mMapId2Cluster[t]
            if cluster not in self.mMapCluster2Colour:
                self.mMapCluster2Colour[cluster] = COLOURS[len(self.mMapCluster2Colour) % len(COLOURS) ]

            colour = self.mMapCluster2Colour[cluster]
            elements.append( SVGdraw.rect( x, y + map_node2height[n] - self.mBoxWidth / 2, self.mBoxWidth, self.mBoxWidth,
                                           stroke = "rgb(%i,%i,%i)" % colour,
                                           fill = "rgb(%i,%i,%i)" % colour) )
                
        return elements

###################################################################
class BranchDecorator:
    """class for decorating the border in a tree plot."""
    def __init__(self, tree):
        self.mTree = tree
        self.mFontSize = 36
        self.mFont = "Verdana"
        self.mFontColour = BLACK
        
    def getHeight( self, node_id ):
        return 1

    def getWidth( self, node_id ):
        return 1

    def setFontSize( self, size ):
        """set font size."""
        self.mFontSize = size

    def setFontColour( self, colour ):
        self.mFontColour = colour
        
class BranchDecoratorVertical( BranchDecorator ):

    
    def __init__(self, *args, **kwargs):
        BranchDecorator.__init__( self, *args, **kwargs)

    def getElements( self, node_id, x, y1, y2 ):

        e = []
        e.append(SVGdraw.line( x, y1, x, y2,
                                stroke = "rgb(%i,%i,%i)" % BLACK,
                                stroke_width = 1 ))
        return e

        
class BranchDecoratorHorizontal( BranchDecorator ):
    """default branch decorator - default is plane line."""
    
    def __init__(self, *args, **kwargs):
        BranchDecorator.__init__( self, *args, **kwargs)
        self.mNoBranch = False
        
    def setNoBranch( self ):
        self.mNoBranch = True
        
    def getElements( self, node_id, x1, x2, y ):

        e = []
        if not self.mNoBranch:
            e.append(SVGdraw.line( x1, y, x2, y,
                                   stroke = "rgb(%i,%i,%i)" % BLACK,
                                   stroke_width = 1 ) )
        return e

class BranchDecoratorHorizontalBranchLength( BranchDecoratorHorizontal ):
    """branch decorator - adds branch length to branch."""

    mBranchLengthFormat = "%5.2f"
    
    def __init__(self, *args, **kwargs):
        BranchDecoratorHorizontal.__init__( self, *args, **kwargs)
        self.mWritten = 0
        
    def getElements( self, node_id, x1, x2, y ):

        e = BranchDecoratorHorizontal.getElements( self, node_id, x1, x2, y )

        text = self.getText( node_id )

        d = x2 - x1
        df = len(text) * (self.mFontSize / 2)
        if df > d:
            x = x1 - df
        else:
            x = x1 + (x2 - x1 - df) / 2

        e.append( SVGdraw.text( x, y - 5,
                                text,
                                self.mFontSize,
                                self.mFont,
                                stroke = "rgb(%i,%i,%i)" % self.mFontColour,
                                text_anchor = "left" ))

        return e

    def getText( self, node_id ):
        """return text."""        
        return self.mBranchLengthFormat % self.mTree.node(node_id).data.branchlength

    def getHeight( self, node_id ):
        return 5 + self.mFontSize
        
class BranchDecoratorHorizontalBranchLengthError( BranchDecoratorHorizontalBranchLength ):
    """branch decorator - adds branch length to branch + error."""

    mBranchErrorFormat = "%5.2f"

    def __init__(self, tree, error_tree, *args, **kwargs):
        BranchDecoratorHorizontalBranchLength.__init__( self, tree, *args, **kwargs)
        self.mErrorTree = error_tree
        
    def getText( self, node_id ):
        """return text."""
        return BranchDecoratorHorizontalBranchLength.getText(self, node_id ) +\
               "+/-" + self.mBranchErrorFormat % self.mErrorTree.node(node_id).data.branchlength        

class BranchDecoratorHorizontalBranchLengthWithKaks( BranchDecoratorHorizontalBranchLength ):
    """branch decorator - adds branch length to branch + kaks."""

    mBranchKaksFormat = "%.2f"

    def __init__(self, tree, error_tree, *args, **kwargs):
        BranchDecoratorHorizontalBranchLength.__init__( self, tree, *args, **kwargs)
        self.mKaksTree = error_tree
        
    def getText( self, node_id ):
        """return text."""
        d = self.mKaksTree.node(node_id).data.branchlength
        if d:
            text = " (%s)" % (self.mBranchKaksFormat % self.mKaksTree.node(node_id).data.branchlength)
        else:
            text = ""

        return BranchDecoratorHorizontalBranchLength.getText(self, node_id ) + text

class BranchDecoratorHorizontalAboveBelow( BranchDecoratorHorizontalBranchLength ):
    """branch decorator - displays two branch values.

    One decorator is displayed above and the the other one below the branch.
    The branch itself is rendered as a horizontal line.
    """

    def __init__(self, tree, decorator1, decorator2, *args, **kwargs):

        BranchDecoratorHorizontalBranchLength.__init__( self, tree, *args, **kwargs)
        self.mDecorator1 = decorator1
        self.mDecorator2 = decorator2

        ## do not write branch
        self.mDecorator1.setNoBranch()
        self.mDecorator2.setNoBranch()        
        
    def getElements( self, node_id, x1, x2, y ):

        e = []
        e.append(SVGdraw.line( x1, y, x2, y,
                               stroke = "rgb(%i,%i,%i)" % BLACK,
                               stroke_width = 1 ) )
        
        e += self.mDecorator1.getElements( node_id, x1, x2, y )
        e += self.mDecorator2.getElements( node_id, x1, x2, y + self.mFontSize )        
        
        return e

###################################################################
class NodeDecorator:
    """class for decorating nodes.

    The default decorator does nothing. It implements the
    methods to plot symbols. Symbol choices are

    * square
    * circle
    
    """

    mFontSize  = 36
    mFont      = "Verdana"
    mFontStyle = None
    
    def __init__(self, tree,
                 plot_label = False,
                 plot_symbol = None):
        
        self.mTree = tree
        self.mPlotLabel = plot_label
        self.mPlotSymbol = plot_symbol

    def getColour( self, node_id, x, y ):
        return BLACK
    
    def getElements( self, node_id, x, y ):

        e = []
        colour = self.getColour( node_id, x, y )
        
        if self.mPlotSymbol == "circle":
            e.append( SVGdraw.circle( x + self.mFontSize / 2,
                                      y,
                                      self.mFontSize/2,
                                      stroke = "rgb(%i,%i,%i)" % BLACK,
                                      fill = "rgb(%i,%i,%i)" % colour) )

        elif self.mPlotSymbol == "square":
            e.append( SVGdraw.rect( x, y-self.mFontSize/2,
                                    self.mFontSize,
                                    self.mFontSize,
                                    stroke = "rgb(%i,%i,%i)" % BLACK,
                                    fill = "rgb(%i,%i,%i)" % colour) )
            
        return e

    def getHeight( self, node_id ):
        if self.mPlotSymbol != None:
            return self.mFontSize
        else:
            return 0
        
    def getWidth( self, node_id ):
        if self.mPlotSymbol != None:
            return self.mFontSize
        else:
            return 0

    def setPlotLabel( self, plot_label ):
        self.mPlotLabel = plot_label

    def getLegend( self, x, y ):
        """add legend.
        returns elements and maximum x and y output.
        """
        return [], x, y

    def getFooterHeight( self ):
        return 0

    def setFontStyle( self, style ):
        self.mFontStyle = style
    
###################################################################        
class NodeDecoratorExternal(NodeDecorator):
    """class for decorating external nodes. Simply write name.
    """
    
    def __init__(self, *args, **kwargs):
        NodeDecorator.__init__( self, *args, **kwargs)
    
    def getElements( self, node_id, x, y, x_label = None, y_label = None ):
        
        e = NodeDecorator.getElements( self, node_id, x, y )

        if x_label == None: x_label = x
        if y_label == None: y_label = y        
        e.append( SVGdraw.text( x_label, y_label,
                                self.mTree.node(node_id).data.taxon,
                                self.mFontSize,
                                self.mFont,
                                stroke = "rgb(%i,%i,%i)" % BLACK,
                                font_style = self.mFontStyle,
                                text_anchor = "left" ))
        return e

    def getHeight( self, node_id ):
        return max( NodeDecorator.getHeight(self, node_id), self.mFontSize)

    def getWidth( self, node_id ):
        m = NodeDecorator.getWidth( self, node_id )
        if self.mPlotLabel:
            l = len(self.mTree.node(node_id).data.taxon) * self.mFontSize
        else:
            l = 0
        return max(l, m)
    
###################################################################
class NodeDecoratorBySpecies(NodeDecorator):
    """class for decorating external nodes.

    Colour by species and rename.
    """

    def __init__(self, tree,
                 plot_label = None,
                 plot_symbol = None,
                 extract_species = str,
                 map_species2colour = None,
                 map_species2name = None,
                 map_taxon2url = None,
                 *args, **kwargs):
        
        """colours are a ','-separated tuples of RGB values.

        """
        ## plot labels unless explicitely set to false:
        if plot_label == None: plot_label = True
        
        NodeDecorator.__init__( self, tree,
                                plot_label = plot_label,
                                plot_symbol = plot_symbol,
                                *args, **kwargs )
        
        self.mExtractSpecies = extract_species
        self.mMapSpecies2Colour = {}

        if map_species2colour:
            for s, c in map_species2colour.items():
                self.mMapSpecies2Colour[s] = tuple(map(int, c.split(",")))

        self.mMapTaxon2URL = map_taxon2url
        if map_species2name:
            self.mMapSpecies2Name = map_species2name
        else:
            self.mMapSpecies2Name = {}

    def getColour( self, node_id, x, y ):

       species = self.mExtractSpecies(self.mTree.node(node_id).data.taxon)
        
       if species not in self.mMapSpecies2Colour:
            self.mMapSpecies2Colour[species] = COLOURS[len(self.mMapSpecies2Colour) % len(COLOURS) ]

       return self.mMapSpecies2Colour[species]
            
    def getElements( self, node_id, x, y, x_label = None, y_label = None):

        e = NodeDecorator.getElements( self, node_id, x, y )

        if x_label == None: x_label = x
        if y_label == None: y_label = y        
        
        t = self.mTree.node(node_id).data.taxon        
        species = self.mExtractSpecies(t)
        
        if species not in self.mMapSpecies2Colour:
            self.mMapSpecies2Colour[species] = COLOURS[len(self.mMapSpecies2Colour) % len(COLOURS) ]

        if species in self.mMapSpecies2Name:
            tx = re.sub( species, "%s" % self.mMapSpecies2Name[species], t)
        else:
            tx = t

        colour = self.getColour( node_id, x, y )
        
        if self.mPlotLabel:
            ee = SVGdraw.text( x_label, y_label,
                               tx,
                               self.mFontSize,
                               self.mFont,
                               stroke = "rgb(%i,%i,%i)" % colour,
                               text_anchor = "left" )

            if self.mMapTaxon2URL != None:
                url = self.mMapTaxon2URL(t)
                if url:
                    l = SVGdraw.link( url )
                    l.addElement( ee )
                    e.append( l )
                else:
                    e.append(ee)
            else:
                e.append( ee )
        return e

    def getHeight( self, node_id ):
        return max( NodeDecorator.getHeight(self, node_id), self.mFontSize)

    def getWidth( self, node_id ):
        m = NodeDecorator.getWidth( self, node_id)
        
        if self.mPlotLabel:
            t = self.mTree.node(node_id).data.taxon
            if t in self.mMapSpecies2Name:
                t = self.mMapSpecies2Name[t]
            l = len(t) * self.mFontSize * 2 / 3
        else:
            l = 0
            
        return max(m, l)
        

###################################################################
    
class SVGTree:

    def __init__(self, tree ):

        self.mElements = {}

        self.mTree = tree
        
        ## footer
        self.mFooterFrom = 10        
        self.mFooterFontSize  = 36
        self.mFooterFont      = "Verdana"
        self.mFooter = None
        
        ## Title
        self.mTitleFontSize  = 36
        self.mTitleFont      = "Verdana"
        self.mTitle = None

        ## Row/column labels
        self.mLabelFontSize  = 36
        self.mLabelFont      = "Verdana"

        ## default data area size (without terminal node labels)
        self.mDefaultHeight = 1000
        self.mDefaultWidth = 1000

        ## Row/column labels
        self.mTerminalLabelSeparator = 5
        self.mTerminalLabelOffset    = 5

        ## factor by which to scale branch lengths
        self.mBranchScaleFactor = 0
        ## factor by which to scale heights
        self.mHeightScaleFactor = 0
        
        self.mDecoratorExternalNodes = NodeDecoratorExternal(self.mTree)
        self.mDecoratorInternalNodes = NodeDecorator(self.mTree)

        self.mSeparatorWidth  = self.mLabelFontSize
        self.mSeparatorHeight = 2 * self.mLabelFontSize        

        ## increment of ruler in branch length
        self.mRulerIncrement = 0.1
        self.mRulerFormat = "%5.2f"
        self.mRulerTickSize = 5
        self.mRulerElements = ("ruler","left-ticks", "scale" )
        self.mRulerFontSize = 36
        self.mRulerFont      = "Verdana"

        ## whether or not terminal labels are left justified
        self.mLeftJustifiedExternalNodes = True

        ## decorators for the border
        self.mBorderDecorators= []

        self.mDecoratorVerticalBranches = BranchDecoratorVertical(self.mTree)
        self.mDecoratorHorizontalBranches = BranchDecoratorHorizontal(self.mTree)        


    #####################################################################
    def setRulerElements( self, ruler_elements = [] ):
        self.mRulerElements = ruler_elements
        
    #####################################################################
    def setBranchScale( self, scale ):
        self.mBranchScaleFactor = scale
        
    #####################################################################
    def setHeightScale( self, scale ):
        self.mHeightScaleFactor = scale

    #####################################################################
    def addBorderDecorator( self, decorator ):
        self.mBorderDecorators.append( decorator )
        
    #####################################################################
    def setDecoratorExternalNodes( self, decorator ):
        self.mDecoratorExternalNodes = decorator        

    #####################################################################
    def getDecoratorExternalNodes( self ):
        return self.mDecoratorExternalNodes

    #####################################################################
    def setDecoratorInternalNodes( self, decorator ):
        self.mDecoratorInternalNodes = decorator        

    #####################################################################
    def getDecoratorInternalNodes( self ):
        return self.mDecoratorInternalNodes
    
    #####################################################################
    def setDecoratorHorizontalBranches( self, decorator ):
        self.mDecoratorHorizontalBranches = decorator        

    #####################################################################
    def getDecoratorHorizontalBranches( self ):
        return self.mDecoratorHorizontalBranches

    #####################################################################
    def setDecoratorVerticalBranches( self, decorator ):
        self.mDecoratorVerticalBranches = decorator        

    #####################################################################
    def setDecoratorVerticalBranches( self ):
        return self.mDecoratorVerticalBranches

    ###################################################################
    def setFontSize( self, size ):
        """set font size in all elements."""
        self.mDecoratorInternalNodes.mFontSize = size
        self.mDecoratorExternalNodes.mFontSize = size
        self.mDecoratorVerticalBranches.mFontSize = size
        self.mDecoratorHorizontalBranches.mFontSize = size        
    
    #####################################################################
    def addElement( self, element, plane = 0):
        """add element to list in plane.
        
        The list is later sorted by priority.
        """
        if plane not in self.mElements:
            self.mElements[plane] = []
            
        self.mElements[plane].append( element )

    #####################################################################
    def addElements( self, elements, plane = 0):
        """add multiple elments to a plane."""
        for e in elements:
            self.addElement( e, plane )

    #####################################################################                        
    def initializePlot( self ):
        """set various coordinates in the plot.

        Note:

        Width  = X = coordinate 1
        Height = Y = coordinate 2
        
        """

        self.mNTaxa = len(self.mTree.get_taxa())
        self.mNNodes = max( self.mTree.chain.keys() ) + 1

        self.calculateCoordinates()
        
        self.calculateCanvasSize( )

    #####################################################################
    def getHeaderHeight( self ):
        """return height of header"""
        return 0

    #####################################################################
    def getHeaderWidth( self ):
        """return width of header"""
        return 0
    
    #####################################################################
    def calculateCanvasSize( self ):
        
        ## calculate size of data area
        self.mDataWidth  = max(self.mNodeWidthsEnd ) + max([ self.mDecoratorExternalNodes.getWidth( x ) for x in self.mTree.get_terminals() ])
        self.mDataHeight = self.mMaxNodeHeight

        ## calculate border width
        self.mBorderWidth = 0
        for x in self.mBorderDecorators:
            self.mBorderWidth += x.getWidth()
            
        ## set the page size
        self.mPageWidth  = self.getHeaderWidth() + self.mDataWidth + self.mBorderWidth
        self.mPageHeight = self.getHeaderHeight() + self.mDataHeight + self.getFooterHeight()

        ## patch: white background
##         self.addElement( SVGdraw.rect( 0, 0, self.mPageWidth, self.mPageHeight,
##                                        stroke = "rgb(%i,%i,%i)" % WHITE,
##                                        fill = "rgb(%i,%i,%i)" % WHITE) )

    #####################################################################
    def getFooterHeight( self ):

        footer_height = 0
        if "ruler" in self.mRulerElements:
            footer_height += 2 * self.mRulerTickSize + 1 + self.mSeparatorHeight
        if "scale" in self.mRulerElements:
            footer_height += 2 * self.mRulerTickSize + 1 + self.mSeparatorHeight + self.mRulerFontSize

        return footer_height + \
               self.mDecoratorExternalNodes.getFooterHeight() + \
               self.mDecoratorInternalNodes.getFooterHeight()

    #####################################################################
    def calculateCoordinates( self ):
        
        self.mNodeHeights = [0] * self.mNNodes
        self.mNodeWidthsStart = [0] * self.mNNodes
        self.mNodeWidthsEnd  = [0] * self.mNNodes        

        ## if no scales are given, try to do best fit
        if self.mHeightScaleFactor == 0:
            rescale_height = True
            self.mHeightScaleFactor = 1
        else:
            rescale_height = False

        if self.mBranchScaleFactor == 0:
            rescale_width = True
            self.mBranchScaleFactor = 100
        else:
            rescale_width = False

        ##########################################################
        ## Get Vertical coordinates
        ## Label nodes by their height. Terminal nodes have integer coordinates.
        ## Internal nodes have fractional coordinates (the average between the two
        ## children)
        counter = [0]        
        def updateHeights (node_id):
            l = len(self.mTree.node(node_id).succ)
            if l:
                ## set node height for internal node
                t = 0
                for x in self.mTree.node(node_id).succ:
                    t += self.mNodeHeights[x]
                    # used to use use the following to take into account
                    # the height of symbols. This is wrong and better done by
                    # pre-traversal of the tree
                    # self.mNodeHeights[node_id] = float(t) / float(l) + max( self.mDecoratorInternalNodes.getHeight( node_id ), self.mDecoratorHorizontalBranches.getHeight( node_id ))
                    # instead: use uncorrected heights.
                    self.mNodeHeights[node_id] = float(t) / float(l) 
            else:
                ## set node height for external node
                self.mNodeHeights[node_id] = counter[0]
                counter[0] += max( self.mDecoratorExternalNodes.getHeight( node_id ),
                                   self.mDecoratorHorizontalBranches.getHeight( node_id) ) \
                                   * self.mHeightScaleFactor + self.mTerminalLabelSeparator
                
        TreeTools.TreeDFS( self.mTree, self.mTree.root,
                           post_function = updateHeights )

        self.mMaxNodeHeight = counter[0]

        ##########################################################
        ## Get horizontal coordinates
        def updateWidths( node_id ):

            node = self.mTree.node( node_id )
            d = node.data.branchlength
            ## set default branchlength to 0.01 for empty branch lengths
            ## TODO: deal with trees without branch lengths later.
            if d <= 0.0: d = 0.01
            right = self.mNodeWidthsStart[node_id] + int(d * self.mBranchScaleFactor)
            self.mNodeWidthsEnd[node_id] = right
            for s in node.succ:
                self.mNodeWidthsStart[s] = right

        TreeTools.TreeDFS( self.mTree, self.mTree.root,
                           pre_function = updateWidths )

        if rescale_height:
            m = max( self.mNodeHeights )
            f = float(self.mDefaultHeight) / m
            if 100 * f < 1: f = 0.01
            self.mHeightScaleFactor = 100 * f
            self.mNodeHeights = map(lambda x: int(x * f), self.mNodeHeights)
            self.mMaxNodeHeight *= f
            
        if rescale_width:
            m = max( self.mNodeWidthsEnd )
            f = float(self.mDefaultWidth) / m
            self.mBranchScaleFactor = 100 * f            
            self.mNodeWidthsStart = map(lambda x: int(x * f), self.mNodeWidthsStart)
            self.mNodeWidthsEnd = map(lambda x: int(x * f), self.mNodeWidthsEnd)            

        ## add a safety margin for decorators writing above the line. This
        ## is a patch and should be changed such that decorators report
        ## their correct height

        for x in range(self.mNNodes):
            self.mNodeHeights[x] += 45

    #####################################################################
    def plotLines( self ):
        """plot lines of the tree"""
        
        ## plot tree in dfs manner
        def plotLines( node_id ):

            node = self.mTree.node( node_id )

            left = self.mNodeWidthsStart[node_id]
            right = self.mNodeWidthsEnd[node_id]
            height = self.mNodeHeights[node_id] 

            if right != left and node_id != self.mTree.root:
                self.addElements( self.mDecoratorHorizontalBranches.getElements(
                    node_id,
                    self.getHeaderWidth() + left,
                    self.getHeaderWidth() + right,
                    self.getHeaderHeight() + height ))
                                 

            for s in node.succ:

                new_height = self.mNodeHeights[s]
                self.addElements( self.mDecoratorVerticalBranches.getElements(
                    node_id,
                    self.getHeaderWidth() + right,
                    self.getHeaderHeight() + height,
                    self.getHeaderHeight() + new_height ))
                
        TreeTools.TreeDFS( self.mTree, self.mTree.root,
                           pre_function = plotLines )

        
    #####################################################################        
    def plotTree( self ):
        """plot the tree."""

        self.plotLines()
        
        self.plotExternalNodes()
        self.plotInternalNodes()        
        
    #####################################################################
    def plotExternalNodes( self ):
        """plot terminal node labels."""

        max_x = max(self.mNodeWidthsEnd)
        for node_id in self.mTree.get_terminals():

            node = self.mTree.node( node_id )

            x = self.mNodeWidthsEnd[node_id]
            y = self.mNodeHeights[node_id]
            
            if self.mLeftJustifiedExternalNodes:
                x_label = max_x
            else:
                x_label = x
                
            e = self.mDecoratorExternalNodes.getElements( node_id,
                                                          self.getHeaderWidth() + x,
                                                          self.getHeaderHeight() + y,
                                                          self.getHeaderWidth() + x_label,
                                                          self.getHeaderHeight() + y )
            
            self.addElements(e)

    #####################################################################
    def plotInternalNodes( self ):
        """plot terminal node labels."""

        
        for node_id in self.mTree.chain.keys():

            node = self.mTree.node( node_id )
            if node.succ == []: continue
            
            x = self.mNodeWidthsEnd[node_id]
            y = self.mNodeHeights[node_id]

            e = self.mDecoratorInternalNodes.getElements( node_id,
                                                          self.getHeaderWidth() + x,
                                                          self.getHeaderHeight() + y )
            
            self.addElements(e)
        
    #####################################################################
    def setTitle( self, title ):
        """set title."""
        self.mTitle = title

    #####################################################################
    def setFooter( self, footer ):
        """set footer."""
        self.mFooter = footer
        
    #####################################################################
    def writeTitle( self ):
        """write title into plot."""
        
        if self.mTitle:
            e = SVGdraw.text( self.mPageWidth / 2,
                              self.mTitleFontSize ,
                              self.mTitle,
                              self.mTitleFontSize,
                              self.mTitleFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.addElement(e)

    #####################################################################
    def writeFooter( self ):
        """write footer.

        The footer contains the legend.
        """
        total_branch_length = (max(self.mNodeWidthsEnd) - min(self.mNodeWidthsStart)) / self.mBranchScaleFactor
        
        self.mFooterX = self.getHeaderWidth()
        self.mFooterY = self.getHeaderHeight() + self.mDataHeight + self.mSeparatorHeight

        ruler_start = self.mFooterX
        ruler_end = self.mFooterX + int(total_branch_length * self.mBranchScaleFactor)

        if "ruler" in self.mRulerElements:
            ## full length ruler with tick marks and labels
            e = SVGdraw.line( ruler_start,
                              self.mFooterY + self.mRulerTickSize + 1,
                              ruler_end,
                              self.mFooterY + self.mRulerTickSize + 1,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              stroke_width = 1 )
            self.addElement( e )

            ## get reasonable intervalls

        increment = self.mRulerIncrement * self.mBranchScaleFactor

        ## adjust increment for extremely long trees
        if (ruler_end - ruler_start) / increment > 1000:
            increment = (ruler_end - ruler_start) / 1000.0
            self.mRulerIncrement = increment / self.mBranchScaleFactor
        
        if "right-ticks" in self.mRulerElements:

            x = ruler_end
            while x >= ruler_start:
                e = SVGdraw.line( x,
                                  self.mFooterY,
                                  x,
                                  self.mFooterY + 2 * self.mRulerTickSize + 1,
                                  stroke = "rgb(%i,%i,%i)" % BLACK,
                                  stroke_width = 1 )
                self.addElement( e )
                x -= self.mRulerIncrement * self.mBranchScaleFactor
                
            self.mFooterY += 2 * self.mRulerTickSize + 1 + self.mSeparatorHeight

        if "left-ticks" in self.mRulerElements:
            
            x = ruler_start
            while x <= ruler_end:
                e = SVGdraw.line( x,
                                  self.mFooterY,
                                  x,
                                  self.mFooterY + 2 * self.mRulerTickSize + 1,
                                  stroke = "rgb(%i,%i,%i)" % BLACK,
                                  stroke_width = 1 )
                self.addElement( e )
                x += increment
                
            self.mFooterY += 2 * self.mRulerTickSize + 1 + self.mSeparatorHeight

        if "scale" in self.mRulerElements:

            w = int(self.mRulerIncrement * self.mBranchScaleFactor)
            
            e = SVGdraw.line( ruler_end,
                              self.mFooterY + self.mRulerTickSize + 1,
                              ruler_end - w,
                              self.mFooterY + self.mRulerTickSize + 1,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              stroke_width = 1 )
            
            self.addElement( e )
            
            e = SVGdraw.line( ruler_end,
                              self.mFooterY,
                              ruler_end,
                              self.mFooterY + 2 * self.mRulerTickSize + 1,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              stroke_width = 1 )
            
            self.addElement( e )

            e = SVGdraw.line( ruler_end - w,
                              self.mFooterY,
                              ruler_end - w,
                              self.mFooterY + 2 * self.mRulerTickSize + 1,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              stroke_width = 1 )
            
            self.addElement( e )

            e = SVGdraw.text( ruler_end - w / 2,
                              self.mFooterY + 2 * self.mRulerTickSize + 1 + self.mRulerFontSize ,
                              self.mRulerFormat % self.mRulerIncrement,
                              self.mRulerFontSize,
                              self.mRulerFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.addElement(e)
            self.mFooterY += 2 * self.mRulerTickSize + 1 + self.mRulerFontSize 
            
        e, self.mFooterX, self.mFooterY = self.mDecoratorExternalNodes.getLegend( self.mFooterX, self.mFooterY )
        self.addElements(e)
        e, self.mFooterX, self.mFooterY = self.mDecoratorInternalNodes.getLegend( self.mFooterX, self.mFooterY )
        self.addElements(e)

    #####################################################################        
    def finalizePlot( self ):
        """build plot."""

        self.plotTree()

        x = self.mDataWidth
        for b in self.mBorderDecorators:
            self.addElements( b.getElements( x, self.getHeaderHeight(), self.mNodeHeights ) )
            x += b.getWidth()
                              
        # self.writeTitle()
        
        self.writeFooter()

        
    #####################################################################        
    def writeToFile(self, outfile):
        """write svg image to file.
        """
        self.finalizePlot()

        kk = self.mElements.keys()
        kk.sort()
        kk.reverse()

        ## make sure the image size is ok
        min_x, min_y, max_x, max_y = 0, 0, 0, 0
        
        for k in kk:
            for e in self.mElements[k]:
                for x in ('x', 'x2', 'x1'):
                    if x in e.attributes:
                        v = e.attributes[x] 
                        min_x = min(min_x, v )
                        max_x = max(max_x, v )
                for y in ('y', 'y2', 'y1'):
                    if y in e.attributes:
                        v = e.attributes[y]
                        min_y = min(min_y, v )
                        max_y = max(max_y, v )

        min_x, min_y = int(math.floor(min_x)), int(math.floor(min_y))
        max_x, max_y = int(math.floor(max_x)), int(math.floor(max_y))        

        for k in kk:
            for e in self.mElements[k]:
                for x in ('x', 'x2', 'x1'):
                    if x in e.attributes:
                        e.attributes[x] -= min_x
                for x in ('y', 'y2', 'y1'):
                    if y in e.attributes:
                        e.attributes[y] -= min_y

        ## now add all the elements
        self.mRoot = SVGdraw.drawing()
        self.mDraw = SVGdraw.svg( (0, 0, self.mPageWidth - min_x, self.mPageHeight - min_y ) , "100%", "100%" )
                        
        for k in kk:
            for e in self.mElements[k]:
                self.mDraw.addElement( e )
            
        self.mRoot.setSVG(self.mDraw)

        tfile = tempfile.mktemp()
        
        self.mRoot.toXml( tfile )

        lines = open(tfile,"r").readlines()
        
        outfile.write(string.join(lines,""))
        outfile.write("\n")
        
        os.remove(tfile)

    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: SVGTree.py 2784 2009-09-10 11:41:14Z andreas $")

    parser.add_option("-i", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option("-s", "--filename-tree", dest="filename_tree", type="string",
                      help="filename with tree."  )
    parser.add_option("-t", "--tree", dest="tree", type="string",
                      help="tree."  )
    parser.add_option( "-r", "--species-regex", dest="species_regex", type="string" ,
                       help="regular expression to extract species from identifier.")
    parser.add_option( "--colour-by-species", dest="colour_by_species", action="store_true",
                      help="colour by species."  )

    parser.add_option( "--branch-scale", dest="branch_scale", type="float",
                      help="branch length scale factor."  )
    parser.add_option( "--height-scale", dest="height_scale", type="float",
                      help="height scale factor."  )
    
    parser.set_defaults(
        titles = "",
        title = "",
        footer = "",
        filename_tree = None,
        species_regex ="^([^|]+)\|",
        colour_by_species = None,
        tree = None,
        branch_scale = 0,
        height_scale = 0,
        )

    (options, args) = Experiment.Start( parser, add_pipe_options = True )

    if options.filename_tree:
        tree_lines = open(options.filename_tree, "r").readlines()
    elif options.tree:
        tree_lines = options.tree
    else:
        raise "please supply a species tree."

    nexus = TreeTools.Newick2Nexus( tree_lines )
    Tree.updateNexus( nexus )
    tree = nexus.trees[0]

    if options.loglevel >= 2:
        tree.display()
    
    plot = SVGTree( tree )

    plot.setBranchScale( options.branch_scale )
    plot.setHeightScale( options.height_scale )    
    
    if options.colour_by_species:
        rx = re.compile(options.species_regex)
        extract_species = lambda x: rx.search( x ).groups()[0]
        plot.setDecoratorExternalNodes( NodeDecoratorBySpecies( tree, extract_species= extract_species ) )
    
    plot.initializePlot()
    
    plot.writeToFile(sys.stdout)
    
    Experiment.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
png2svg.py - arrange several bitmat images into a single image
==============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes a collection of .png files and outputs an .svg file
arranging the images side by side or in different layers.

Usage
-----

Example::

   python png2svg.py --help

Type::

   python png2svg.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess
import random

from types import *
""" program $Id: png2svg.py 2782 2009-09-10 11:40:29Z andreas $

arrange png files in an svg image.
"""
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.SVGdraw as SVGdraw

###------------------------------------------------------------------------------------------------

###################################################################

class Image:

    def __init__(self, filename, x, y, width, height, opacity = 1.0 ):
        self.mFilename = filename
        self.mWidth = width
        self.mHeight = height
        self.mX = x
        self.mY = y

        self.mOpacity = opacity

    def getSVG( self, xoffset, yoffset ):
        return SVGdraw.image( self.mFilename,
                              self.mX + xoffset,
                              self.mY + yoffset,
                              self.mWidth,
                              self.mHeight,
                              opacity = self.mOpacity )
    
class PngPlot:

    def __init__(self, filenames ):

        self.mElements = {}

        self.mFilenames = filenames
        
        ## footer
        self.mFooterFrom = 10        
        self.mFooterFontSize  = 36
        self.mFooterFont      = "Verdana"
        self.mFooter = None
        
        ## Title
        self.mTitleFontSize  = 36
        self.mTitleFont      = "Verdana"
        self.mTitle = None

        ## Row/column labels
        self.mLabelFontSize  = 36
        self.mLabelFont      = "Verdana"

        ## default data area size (without terminal node labels)
        self.mDefaultHeight = 1000
        self.mDefaultWidth = 1000

        self.mSeparatorWidth  = 10
        self.mSeparatorHeight = 10

        self.mKeepAspectRatio = True

        self.mMapImage2Coords = []

    #####################################################################
    def setCanvasSize( self, width, height ):
        """set canvas width and size."""
        self.mDefaultWidth, self.mDefaultHeight = width, height

    #####################################################################
    def addElement( self, element, plane = 0):
        """add element to list in plane.
        
        The list is later sorted by priority.
        """
        if plane not in self.mElements:
            self.mElements[plane] = []
            
        self.mElements[plane].append( element )

    def getHeaderHeight(self):
        return 0
    def getHeaderWidth(self):
        return 0
    def getFooterHeight(self):
        return 0
    def getFooterWidth(self):
        return 0

    #####################################################################
    def addElements( self, elements, plane = 0):
        """add multiple elments to a plane."""
        for e in elements:
            self.addElement( e, plane )

    #####################################################################                        
    def calculateCanvasSize( self ):
        """calculate the size of the canvas."""

        ## set the page size
        self.mPageWidth  = self.getHeaderWidth() + self.mDataWidth + self.getFooterWidth()
        self.mPageHeight = self.getHeaderHeight() + self.mDataHeight + self.getFooterHeight()
    
    #####################################################################                        
    def initializePlot( self ):
        """set various coordinates in the plot.

        Note:

        Width  = X = coordinate 1
        Height = Y = coordinate 2
        
        """

        self.calculateCoordinates()
        
        self.calculateCanvasSize( )

    #####################################################################
    def setTitle( self, title ):
        """set title."""
        self.mTitle = title

    #####################################################################
    def setFooter( self, footer ):
        """set footer."""
        self.mFooter = footer

    #####################################################################
    def writeImages( self ):
        """write images."""

        xoffset = self.getHeaderWidth()
        yoffset = self.getHeaderHeight()
        
        for i in range(len(self.mFilenames) ):
            image = self.mMapImage2Coords[i]
            e = image.getSVG( xoffset, yoffset )
            self.addElement( e )
        
    #####################################################################
    def writeTitle( self ):
        """write title into plot."""
        
        if self.mTitle:
            e = SVGdraw.text( self.mPageWidth / 2,
                              self.mTitleFontSize ,
                              self.mTitle,
                              self.mTitleFontSize,
                              self.mTitleFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.addElement(e)

    #####################################################################
    def writeFooter( self ):
        """write footer.

        The footer contains the legend.
        """
        if self.mTitle:
            e = SVGdraw.text( self.mPageWidth / 2,
                              self.mTitleFontSize ,
                              self.mTitle,
                              self.mTitleFontSize,
                              self.mTitleFont,
                              stroke = "rgb(%i,%i,%i)" % BLACK,
                              text_anchor = "middle" )

            self.addElement(e)

    #####################################################################        
    def finalizePlot( self ):
        """build plot."""

        self.writeImages()
        self.writeTitle()
        self.writeFooter()
        
    #####################################################################        
    def writeToFile(self, outfile):
        """write svg image to file.
        """
        self.finalizePlot()

        kk = self.mElements.keys()
        kk.sort()
        kk.reverse()

        ## make sure the image size is ok
        min_x, min_y, max_x, max_y = 0, 0, 0, 0
        
        for k in kk:
            for e in self.mElements[k]:
                for x in ('x', 'x2', 'x1'):
                    if x in e.attributes:
                        v = e.attributes[x] 
                        min_x = min(min_x, v )
                        max_x = max(max_x, v )
                for y in ('y', 'y2', 'y1'):
                    if y in e.attributes:
                        v = e.attributes[y]
                        min_y = min(min_y, v )
                        max_y = max(max_y, v )

        min_x, min_y = int(math.floor(min_x)), int(math.floor(min_y))
        max_x, max_y = int(math.floor(max_x)), int(math.floor(max_y))        

        for k in kk:
            for e in self.mElements[k]:
                for x in ('x', 'x2', 'x1'):
                    if x in e.attributes:
                        e.attributes[x] -= min_x
                for x in ('y', 'y2', 'y1'):
                    if y in e.attributes:
                        e.attributes[y] -= min_y

        ## now add all the elements
        self.mRoot = SVGdraw.drawing()
        self.mDraw = SVGdraw.svg( (0, 0, self.mPageWidth - min_x, self.mPageHeight - min_y ) , "100%", "100%" )
                        
        for k in kk:
            for e in self.mElements[k]:
                self.mDraw.addElement( e )
            
        self.mRoot.setSVG(self.mDraw)

        tfile = tempfile.mktemp()
        
        self.mRoot.toXml( tfile )

        lines = open(tfile,"r").readlines()
        
        outfile.write(string.join(lines,""))
        outfile.write("\n")
        
        os.remove(tfile)

class Grid( PngPlot ):

    """grid layout."""
    def __init__(self, filenames, force_square = False, *args, **kwargs ):

        PngPlot.__init__(self, filenames, *args, **kwargs )

        self.mForceSquare = force_square
    
    #####################################################################                        
    def calculateCoordinates( self ):
        """calculate coordinates."""

        ## square image
        nimages = len(self.mFilenames)
        nimages_x = int(math.sqrt( float(nimages) * (float(self.mDefaultWidth) / self.mDefaultHeight) ) )
        if self.mForceSquare:
            nimages_y = nimages_x
            del self.mFilenames[nimages_y * nimages_x:]
        else:
            nimages_y = int(math.ceil(float(nimages) / nimages_x))

        image_width  = int(float((self.mDefaultWidth  - (nimages_x * self.mSeparatorWidth)) / nimages_x))
        ## add +1 for image_height, as the we use left/upper coordiantes
        image_height = int(float((self.mDefaultHeight - ((nimages_y) * self.mSeparatorHeight)) / (nimages_y)))

        if self.mKeepAspectRatio:
            m = min(image_width, image_height)
            image_width, image_height = m, m

        coord_y = -(self.mSeparatorHeight + image_height)
        coord_x = 0
        
        for x in range(len(self.mFilenames)):
            if x % nimages_x == 0:
                coord_x = 0
                coord_y += self.mSeparatorHeight + image_height
                
            self.mMapImage2Coords.append( Image(self.mFilenames[x], coord_x, coord_y, image_width, image_height) )
            
            coord_x += self.mSeparatorWidth + image_width 
            
        self.mDataWidth = self.mDefaultWidth
        self.mDataHeight = self.mDefaultHeight

class RandomLayers( PngPlot ):

    """elements are positioned randomly in several layers."""
    
    def __init__(self, filenames,
                 num_layers,
                 image_width = 100,
                 image_height = 100,
                 *args, **kwargs ):

        PngPlot.__init__(self, filenames, *args, **kwargs )

        self.mNumLayers = num_layers
        self.mImageHeight = image_height
        self.mImageWidth = image_width
        
    #####################################################################                        
    def calculateCoordinates( self ):
        """calculate coordinates."""

        nimages = len(self.mFilenames)
        images_per_layer = int(float( nimages ) / self.mNumLayers)
        opacity_intervall = 1.0 / self.mNumLayers

        opacity = 0 
        for x in range(nimages):
            
            if x % images_per_layer == 0:
                opacity += opacity_intervall

            image_width  = self.mImageWidth
            image_height = self.mImageHeight

            coord_x = random.randint( 0, self.mDefaultWidth - image_width )
            coord_y = random.randint( 0, self.mDefaultHeight - image_height )            
            
            self.mMapImage2Coords.append( Image(self.mFilenames[x], coord_x, coord_y, image_width, image_height, opacity = opacity ) )
            
        self.mDataWidth = self.mDefaultWidth
        self.mDataHeight = self.mDefaultHeight
    

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: png2svg.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-i", "--title", dest="title", type="string",
                      help="page title.")
    parser.add_option("-f", "--footer", dest="footer", type="string",
                      help="page footer.")
    parser.add_option("-l", "--layout", dest="layout", type="choice",
                      choices=("grid", "random", "random-layers",),
                      help="layout to choose.")
    parser.add_option( "--num-layers", dest="num_layers", type="int",
                      help="number of layers.")
    parser.add_option( "--canvas-width", dest="canvas_width", type="int",
                      help="canvas width [default: 1000].")
    parser.add_option( "--canvas-height", dest="canvas_height", type="int",
                      help="canvas height [default: 1000].")
    parser.add_option( "--image-width", dest="image_width", type="int",
                      help="image width.")
    parser.add_option( "--image-height", dest="image_height", type="int",
                      help="image height.")
    parser.add_option( "--force-square", dest="force_square", action="store_true",
                      help="force square layout.")

    
    parser.set_defaults(
        titles = "",
        title = "",
        footer = "",
        layout = "grid",
        force_square = False,
        num_layers = 2,
        image_width = 100,
        image_height = 100,
        canvas_width = 1000,
        canvas_height = 1000,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if len(args) > 0:
        ## read filenames from the command line
        filenames = args
    else:
        ## read filenames from stdin
        filenames = map(lambda x: x[:-1], sys.stdin.readlines())

    if options.loglevel >= 1:
        options.stdlog.write("# arranging %i images.\n" % len(filenames))

    if options.layout == "grid":
        plot = Grid( filenames, force_square = options.force_square )
    elif options.layout == "random":
        plot = RandomLayers( filenames,
                             num_layers = 1,
                             image_width = options.image_width,
                             image_height = options.image_height)
    elif options.layout == "random-layers":
        plot = RandomLayers( filenames,
                             num_layers = options.num_layers,
                             image_width = options.image_width,
                             image_height = options.image_height)
    else:
        raise "unknown layout %s" % options.layout

    plot.setCanvasSize( options.canvas_width, options.canvas_height )
    
    plot.initializePlot()

    plot.writeToFile(sys.stdout)
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
data2histogram.py - histogram data in a table
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes histograms over one or more
columns of a table.

Usage
-----

Example::

   python data2histogram.py --help

Type::

   python data2histogram.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import types

import CGAT.Experiment as E
import CGAT.Histogram as Histogram
import numpy

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: data2histogram.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-r", "--range", dest="range", type="string",
                      help="range to calculate histogram for."  )
    parser.add_option("-b", "--bin-size", dest="bin_size", type="string",
                      help="bin size."  )
    parser.add_option("-i", "--titles", dest ="titles", action="store_true",
                      help="use supplied column titles." )
    parser.add_option( "--no-titles", dest ="titles", action="store_false",
                      help="no column titles given." )
    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to take for calculating histograms." )
    parser.add_option("--min-data", dest="min_data", type="int",
                      help="minimum amount of data required, if less data, then the histogram will be empty [default=%default].")
    parser.add_option("--min-value", dest="min_value", type="float",
                      help="minimum value for histogram.")
    parser.add_option("--max-value", dest="max_value", type="float",
                      help="maximum value for histogram.")
    parser.add_option("--no-empty-bins", dest="no_empty_bins", action="store_true",
                      help="do not display empty bins.")
    parser.add_option("--with-empty-bins", dest="no_empty_bins", action="store_false",
                      help="display empty bins.")
    parser.add_option("--normalize", dest="normalize", action="store_true",
                      help="normalize histogram.")
    parser.add_option("--cumulative", dest="cumulative", action="store_true",
                      help="calculate cumulative histogram.")
    parser.add_option("--reverse-cumulative", dest="reverse_cumulative", action="store_true",
                      help="calculate reverse cumulative histogram.")
    parser.add_option("--headers", dest="headers", type="string",
                      help="use the following headers.")
    parser.add_option("--ignore-out-of-range", dest="ignore_out_of_range", action="store_true",
                      help="ignore values that are out of range (as opposed to truncating them to range border.")
    parser.add_option("--missing", dest="missing_value", type="string",
                      help="entry for missing values [%default]." )
    parser.add_option("--dynamic-bins", dest="dynamic_bins", action="store_true",
                      help="each value constitutes its own bin." )
    parser.add_option("--on-the-fly", dest="on_the_fly", action="store_true",
                      help="on the fly computation of histograms. Requires setting of min-value, max-value and bin_size." )
    
    parser.set_defaults(
        bin_size = None,
        range = None,
        titles = True,
        columns = "all",
        append = (),
        no_empty_bins = True,
        min_value = None,
        max_value = None,
        normalize = False,
        cumulative = False,
        reverse_cumulative = False,
        nonull = None,
        ignore_out_of_range = False,
        min_data = 1,
        headers = None,
        missing_value = "na",
        dynamic_bins = False,
        on_the_fly = False,
        bin_format = "%.2f",
        value_format = "%6.4f",
        )

    (options, args) = E.Start( parser )

    if options.columns != "all":
        options.columns = map(lambda x: int(x) -1 , options.columns.split(","))

    if options.range:
        options.min_value, options.max_value = map(float, options.range.split(","))

    if options.headers:
        options.headers = options.headers.split(",")
        
    if options.on_the_fly:
        if options.min_value == None or options.max_value == None or options.bin_size == None:
            print USAGE
            raise "please supply columns, min-value, max-value and bin-size for on-the-fly computation."

        # try to glean titles from table:
        if options.titles:
            while 1:
                line = sys.stdin.readline()
                if not line: break
                if line[0] == "#": continue
                data = line[:-1].split("\t")
                break

            if options.columns == "all":
                options.titles = data
                options.columns = range( len(data) )
            else:
                options.titles = [ data[x] for x in options.columns ]
                
        bins = numpy.arange( options.min_value, options.max_value, float(options.bin_size))
        hh = Histogram.fillHistograms( sys.stdin, options.columns, [ bins for x in range(len(options.columns) ) ] )
        n = len(hh)

        titles = ['bin']

        if options.headers:
            titles.append( options.headers[x] )
        elif options.titles:
            titles.append( options.titles[x] )
        else:
            for x in options.columns:
                titles.append( "col%i" % (x+1) )

        if len(titles) > 1:
            options.stdout.write( "\t".join(titles) + "\n" )
        
        for x in range(len(bins)):
            v = []
            v.append( options.bin_format % bins[x] )
            for c in range(n):
                v.append( options.value_format % hh[c][x] )

            options.stdout.write( "\t".join( v ) + "\n" )

    else:
        ## in-situ computation of histograms
        # retrieve data
        first = True
        vals = []

        # parse data, convert to floats
        for l in options.stdin:

            if l[0] == "#": continue

            data = string.split(l[:-1], "\t")

            if first:
                first = False
                ncols = len(data)
                if options.columns == "all":
                    options.columns = range(ncols)

                vals = [ [] for x in options.columns ]

                if options.titles:
                    try:
                        options.titles = [ data[x] for x in options.columns ]
                    except IndexError:
                        raise IndexError, "not all columns %s found in data %s" % (str(options.columns), str(data))
                    continue

            for x in range(len(options.columns)):

                try:
                    v = string.atof(data[options.columns[x]])
                except IndexError:
                    print "# IndexError in line:", l[:-1]
                    continue
                except ValueError:
                    continue

                vals[x].append( v )

        lines = None

        hists = []
        titles = []
        
        if not vals:
            if options.loglevel >= 1:
                options.stdlog.write( "# no data\n" )
            E.Stop()
            sys.exit(0)

        for x in range(len(options.columns)):

            if options.loglevel >= 1:
                options.stdlog.write( "# column=%i, num_values=%i\n" % (options.columns[x], len(vals[x])) )

            if len(vals[x]) < options.min_data: continue

            h = Histogram.Calculate( vals[x],
                                     no_empty_bins = options.no_empty_bins,
                                     increment = options.bin_size,
                                     min_value = options.min_value,
                                     max_value = options.max_value,
                                     dynamic_bins = options.dynamic_bins,
                                     ignore_out_of_range = options.ignore_out_of_range )

            if options.normalize: h = Histogram.Normalize( h )
            if options.cumulative: h = Histogram.Cumulate( h )
            if options.reverse_cumulative: h = Histogram.Cumulate( h, direction = 0 )

            hists.append(h)

            for m in options.append:
                if m == "normalize":
                    hists.append( Histogram.Normalize( h ) )

            if options.headers:
                titles.append( options.headers[x] )
            elif options.titles:
                titles.append( options.titles[x] )
            else:
                titles.append( "col%i" % options.columns[x] )

        if titles:
            options.stdout.write( "bin\t" + "\t".join(titles) + "\n" )

        if len(hists) == 1:
            Histogram.Print( hists[0], nonull = options.nonull )
        else:
            combined_histogram = Histogram.Combine( hists, missing_value = options.missing_value )
            Histogram.Print( combined_histogram, nonull = options.nonull )        

    E.Stop()









#cimport csamtools

from pysam.csamtools cimport *
import collections
import CGAT.Experiment as E
import numpy

PeakShapeResult = collections.namedtuple( "PeakShapeResult",
                                          "interval_width npeaks "
                                          "peak_center peak_width peak_height peak_relative_pos "
                                          "nreads "
                                          "median closest_half_height furthest_halfheight "
                                          "bins counts" )

PeakShapeCounts = collections.namedtuple( "PeakShapeCounts",
                                          "nreads median counts" )

cdef class Counter:
    '''base class for counters computing densities 
    from genomic data.
    '''
    def __init__(self):
        pass

    def countAroundPos( self, 
                        infile,
                        contig, 
                        int pos,
                        bins,
                        **kwargs ):
        '''count and bin in bins.

        return a result object.
        '''
        cdef int xstart, xend, rstart, rend, i, offset

        nreads, counts = self.coverageInInterval( infile, 
                                                  contig, 
                                                  max( 0, pos + bins[0]),
                                                  pos + bins[-1],
                                                  **kwargs )

        nbins = len(bins) - 1
        hist = numpy.zeros( nbins, dtype = numpy.int )
        cdef int lcounts = len(counts)

        offset = -bins[0]
        xstart = offset + bins[0] 
        for i from 0 <= i < nbins:
            xend = offset + bins[i+1]
            # only take complete bins
            if xstart >= 0 and xend < lcounts:
                hist[i] = sum( counts[xstart:xend] ) 
            xstart = xend

        result = PeakShapeCounts._make( (nreads, 
                                         numpy.median(counts),
                                         hist ) )

        return result

    def countInInterval( self,
                         infile,
                         contig, 
                         int start, 
                         int end,
                         bins,
                         int window_size = 0,
                         float peak_ratio = 0.90,
                         only_interval = False,
                         centring_method = "reads" ):
        '''count density in window and compute peak-shape 
        summary parameters inside window.

        return a result object.
        '''

        cdef int peak_nreads = 0
        cdef int npeaks = 0
        cdef int peak_center = 0
        cdef int i, x
        cdef int xstart, xend, rstart, rend

        assert start >= 0, "start < 0"

        # maximum extend of window = interval +- window_size
        cdef int maxwindow_start = max(0, start - window_size )
        cdef int maxwindow_end = end + window_size

        # bases added at right/left of interval
        cdef int offset_right = maxwindow_end - end
        cdef int offset_left = start - maxwindow_start

        cdef int interval_width = end - start

        # get counts in window
        nreads, counts_in_window = self.coverageInInterval( infile, 
                                                            contig, 
                                                            maxwindow_start, 
                                                            maxwindow_end )

        # counts only in interval - used to define peak center
        counts_in_interval = counts_in_window[offset_left:-offset_right]

        if len(counts_in_interval) == 0:
            raise ValueError( "empty interval: %i - %i for %s:%i-%i" % (offset_left, -offset_right, contig, start, end) )

        #################################################
        # compute peak shape parameters
        peak_nreads = max(counts_in_interval)
        peaks = numpy.array( range(0,interval_width) )[ counts_in_interval >= peak_nreads ]
        if centring_method == "reads":
            peak_center = peaks[len(peaks) // 2] 
        elif centring_method == "middle":
            peak_center = interval_width // 2
        else:
            raise ValueError( "unknown centring method '%s'" % centring_method)

        # define peak height
        cdef int peak_height = numpy.ceil( peak_ratio * peak_nreads )

        peaks = numpy.array( range(0,interval_width) )[ counts_in_interval >= peak_height ]
        npeaks = len(peaks)
        peak_start = peaks[0]
        peak_end = peaks[-1]
        cdef int peak_width = peak_end - peak_start

        # find size of peak
        # cdef int peak_start, peak_end
        # peak_start, peak_end = peak_center, peak_center

        # for peak_center >= i >= 0:
        #     if counts_in_interval[i] < peak_height:
        #         peak_start = i
        #         break

        # for peak_center <= i < interval_width:
        #     if counts_in_interval[i] < peak_height:
        #         peak_end = i
        #         break

        # closest and furthest distance of peak to half-height
        cdef int half_height = peak_height // 2
        cdef int left_first, left_last, right_first, right_last 
        left_first, left_last, right_first, right_last = peak_center, 0, interval_width, peak_center

        for i from 0 <= i < peak_center:
            if counts_in_interval[i] >= half_height: 
                left_first = i
                break

        for i from peak_center > i >= 0:
            if counts_in_interval[i] <= half_height: 
                left_last = i
                break

        for i from peak_center < i < interval_width:
            if counts_in_interval[i] <= half_height: 
                right_first = i
                break

        for i from interval_width > i >= peak_center:
            if counts_in_interval[i] >= half_height: 
                right_last = i
                break

        cdef int furthest_dist = max( peak_center - left_first, right_last - peak_center )
        cdef int closest_dist = min( peak_center - left_last, right_first - peak_center )

        #################################################
        # compute histogram
        nbins = len(bins) - 1
        hist = numpy.zeros( nbins, dtype = numpy.int )

        # decide in which region to count - interval or window
        if only_interval:
            counts = counts_in_interval
            # offset = peak
            offset = peak_center
        else:
            counts = counts_in_window
            # offset = peak to its corresponding location in
            # counts_in_window
            offset = peak_center + offset_left

        cdef int lcounts = len(counts)

        xstart = offset + bins[0] 
        for i from 0 <= i < nbins:
            xend = offset + bins[i+1]
            # only take complete bins
            if xstart >= 0 and xend < lcounts:
                hist[i] = sum( counts[xstart:xend] ) 
            xstart = xend

        # debugging
        #for x,v in enumerate( hist ):
            # print x, "*" * v
        # for x,v in enumerate( counts ):
        #     print x, "*" * v

        result = PeakShapeResult._make( (interval_width, npeaks,
                                         start + peak_center, 
                                         peak_width, peak_nreads, 
                                         abs((interval_width // 2) - peak_center), 
                                         nreads,
                                         numpy.median(counts),
                                         closest_dist, furthest_dist,
                                         bins,
                                         hist ) )


        return result


cdef class CounterBam(Counter):
    '''compute densities in intervals from bam files.'''

    cdef int shift

    def __init__(self, shift = 0):

        Counter.__init__(self)

        self.shift = shift

    #################################################
    # bigwig versions
    def coverageInInterval( self,
                             Samfile samfile,
                             contig,
                             int start, 
                             int end ):
        '''return coverage in window on *contig* bounded by *start* and *end*.

        Reads are optionally shifted by shift/2. Reads are shifted upstream
        on the forward strand and downstream on the reverse strand.

        returns a tuple:
           nreads = number of reads/tags counted
           counts = numpy array with reads per base
        '''
        cdef int shift = self.shift

        # for peak counting follow the MACS protocol:
        # see the function def __tags_call_peak in PeakDetect.py
        #
        # In words
        # Only take the start of reads (taking into account the strand)        # add d/2=shift to each side of peak and start accumulate counts.
        # for counting, extend reads by 2 * shift
        # on + strand shift tags upstream
        # i.e. look at the downstream window
        # note: filtering?
        # note: does this work with paired-end data?
        cdef int offset = shift // 2
        cdef int nreads = 0

        # collect pileup profile in region bounded by maxwindow_start and maxwindow_end.
        cdef int interval_width = end - start
        cdef int * ccounts = <int*>calloc( interval_width, sizeof(int))

        if interval_width <= 0: 
            return 0, numpy.zeros(0)

        if shift:
            xstart, xend = max(0, start - offset), max(0, end - offset)

            for read in samfile.fetch( contig, xstart, xend ):
                if read.is_reverse: continue
                nreads += 1
                # extend reads from upstream pos
                rstart = read.pos
                rend = rstart + 2 * offset 
                # truncate to interval
                rstart = max( 0, rstart - xstart )
                rend = min( interval_width, rend - xstart )
                for i from rstart <= i < rend: ccounts[i] += 1

            # on the - strand, shift tags downstream
            xstart, xend = start + offset, end + offset

            for read in samfile.fetch( contig, xstart, xend ):
                if not read.is_reverse: continue
                nreads += 1
                # shift and extend reads from downstream pos
                rend = read.aend
                rstart = rend - 2 * offset
                # truncate to interval
                rstart = max( 0, rstart - xstart )
                rend = min( interval_width, rend - xstart )

                for i from rstart <= i < rend: ccounts[i] += 1

        else:
            for read in samfile.fetch( contig, start, end ):
                nreads += 1
                # truncate to interval
                rstart = max( 0, read.pos - start )
                rend = min( interval_width, read.aend - start )
                for i from rstart <= i < rend: ccounts[i] += 1

        # transfer to numpy count object
        counts = numpy.zeros( interval_width, dtype = numpy.int )
        for i from 0 <= i < interval_width: counts[i] = ccounts[i]

        free( ccounts )

        return nreads, counts

cdef class CounterBigwig(Counter):
    '''compute densities in intervals from bigwig files.'''

    def __cinit__(self, shift = 0):
        Counter.__init__(self)

    def coverageInInterval( self,
                             wigfile,
                             contig,
                             int start, 
                             int end ):
        '''return coverage in window on *contig* bounded by *start* and *end*.

        Reads are optionally shifted by shift/2. Reads are shifted upstream
        on the forward strand and downstream on the reverse strand.

        returns a tuple:
           nreads = number of reads/tags counted
           counts = numpy array with reads per base
        '''
        # collect pileup profile in region bounded by maxwindow_start and maxwindow_end.
        cdef int interval_width = end - start

        if interval_width <= 0: 
            return 0, numpy.zeros(0)

        d = wigfile.summarize( contig, start, end, end - start)
        
        # transfer to numpy count object
        nreads = sum( d.valid_count )

        return nreads, d.sum_data

    




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
calculate_histogram.py - calculate histogram from data
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script calculates histograms from data in a
tab-separated table.

read in data and build histogram of column.

Options::

    -c, --column            column to take [default = 0]
    -a, --append=           append columns [normalize]
    -n, --normalize         normalize column
    --cumulative            cumulative histogram
    --reverse-cumulative    reverse cumulative histogram
    -i, --titles            use supplied titles
    # at start of line is a comment

Usage
-----

Example::

   python calculate_histogram.py --help

Type::

   python calculate_histogram.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time

import CGAT.Experiment as E
import CGAT.Histogram as Histogram

param_loglevel = 1
param_separator = "//"
param_take = None
param_fill = 0
param_nonull = None
param_columns = [0,]
param_empty_bins = 1
param_titles = False

param_lower_limit = None
param_upper_limit = None
param_bin_size = None
param_scale = None
param_normalize = None
param_append = []

param_cumulative = False
param_reverse_cumulative = False

param_long_options = ["Verbose=", "nonull", "fill","take=", "column=", "show_empty",
                      "upper=", "lower=", "bin-size=", "scale=", "normalize", "append=", "titles",
                      "cumulative", "reverse-cumulative", "help",
                      "version"]

param_short_options = "v:nft:c:eu:l:b:a:ioh"

##---------------------------------------------------------------------------------------------------------        
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      param_short_options,
                                      param_long_options)

    except getopt.error, msg:
        print globals()["__doc__"]
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "-h", "--help" ):
            print globals()["__doc__"]
            sys.exit(0)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ("-t", "--take"):
            param_take = map(string.atoi, string.split(a, ","))
        elif o in ("-f", "--fill"):
            param_fill = 1
        elif o in ("-n", "--nonull"):
            param_nonull = ""
        elif o in ("-c", "--column"):
            if a == "all":
                param_columns = "all"
            else:
                param_columns = map(lambda x: int(x) - 1, string.split( a, ","))
        elif o in ("-e", "--show_empty"):
            param_empty_bins = 0
        elif o in ("-u", "--upper"):
            param_upper_limit = string.atof(a)
        elif o in ("-l", "--lower"):
            param_lower_limit = string.atof(a)
        elif o in ("-b", "--bin-size"):
            param_bin_size = string.atof(a)
        elif o in ("-s", "--scale"):
            param_scale = float(a)
        elif o in ("-o", "--normalize"):
            param_normalize = True
        elif o in ("-a", "--append"):
            param_append = string.split( a, ",")
        elif o in ("-i", "--titles"):
            param_titles = True
        elif o == "--cumulative":
            param_cumulative = True
        elif o == "--reverse-cumulative":
            param_reverse_cumulative = True
        
    histograms = []

    if param_loglevel >= 1:
        print E.GetHeader()
        print E.GetParams()    
    
    vals = []
    
    for x in param_columns: vals.append( [] )
    
    # retrieve histogram
    lines = filter( lambda x: x[0] <> "#", sys.stdin.readlines())

    ncols = len(string.split(lines[0][:-1], "\t"))
    if param_columns == "all":
        param_columns = range(ncols)
        for x in param_columns: vals.append( [] )

    if param_titles:
        data = lines[0][:-1].split("\t")
        del lines[0]
        param_titles = map( lambda x: data[x], param_columns)
        
    for l in lines:
        data = string.split(l[:-1], "\t")
            
        for x in range(len(param_columns)):
            try:
                v = string.atof(data[param_columns[x]])
            except IndexError:
                print "# IndexError in line:", l[:-1]
                continue
            except ValueError:
                continue

            if param_scale:
                v *= param_scale

            if param_upper_limit != None and v > param_upper_limit:
                v = param_upper_limit

            if param_lower_limit != None and v < param_lower_limit:
                v = param_lower_limit

            vals[x].append( v )

    lines = None

    hists = []
    titles = []
    
    for x in range(len(param_columns)):
        if param_loglevel >= 1:
            print "# column=%i, num_values=%i" % (param_columns[x], len(vals[x]))

        if len(vals[x]) == 0: continue
        
        h = Histogram.Calculate( vals[x], no_empty_bins = param_empty_bins, increment = param_bin_size)
        if param_scale: h = Histogram.Scale( h, 1.0 / param_scale )

        if param_normalize: h = Histogram.Normalize( h )
        if param_cumulative: h = Histogram.Cumulate( h )
        if param_reverse_cumulative: h = Histogram.Cumulate( h, direction = 0 )
        
        hists.append(h)

        for m in param_append:
            if m == "normalize":
                hists.append( Histogram.Normalize( h ) )

        if param_titles:
            titles.append( param_titles[x] )

    if titles:
        print "bin\t" + "\t".join(titles)

    if len(hists) == 1:
        Histogram.Print( hists[0], nonull = param_nonull )
    else:
        combined_histogram = Histogram.Combine( hists )
        Histogram.Print( combined_histogram, nonull = param_nonull )        

    if param_loglevel >= 1:        
        print E.GetFooter()
    













################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
VCF.py - Tools for working with VCF files
=========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

The Variant Call Format (:term:`vcf`) is described
here:

http://www.1000genomes.org/wiki/doku.php?id=1000_genomes:analysis:vcf4.0


Code
----

'''
import sys, gzip


class VCFEntry:
    def __init__(self, data, samples ):

        assert len(data) == len(samples) + 9
        self.contig, self.pos, self.id, self.ref, self.alt, self.qual, self.filter, self.info, self.format = \
            data[:9]
        
        self.genotypes = dict( zip(samples, data[9:]))
        self.order = samples

    def __str__(self):
        return "\t".join( map(str, (\
                    self.contig, self.pos, self.id, self.ref, self.alt, self.qual, 
                    self.filter, self.info, self.format,
                    "\t".join( [self.genotypes[x] for x in self.order ] ) ) ) )


class VCFFile:

    def __init__(self, infile):

        self.infile = infile
        self.format = {}
        self.info = {}
        self.fileformat = None

        while 1:
            line = self.infile.readline()
            
            if line.startswith("##"): 
                self.addMetaFromLine( line[2:-1] )
                continue
            elif line.startswith("#CHROM"):
                self.samples = line[:-1].split("\t")[9:]
                continue
            elif line.startswith("#"):
                continue
            break
        self.line = line

    def writeHeader( self, outfile, order = None ):
        outfile.write("##fileformat=%s\n" % self.fileformat)
        for key,values in self.format.iteritems():
            outfile.write("##FORMAT=%s,%s\n" % (key, ",".join(values)))
        for key,values in self.info.iteritems():
            outfile.write("##INFO=%s,%s\n" % (key, ",".join(values)))
        outfile.write("#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t" )
        if order:
            assert len(order) == len(self.samples), \
                "number of samples do not match: %i != %i" % (len(order), len(self.samples))
            outfile.write("\t".join(order))
        else:
            outfile.write("\t".join(self.samples))
        outfile.write("\n")

    def __iter__( self ):
        return self

    def addMetaFromLine( self, line ):

        key, value = line.split("=",1)
        if key == "INFO": 
            data = value.split(",")
            self.info[data[0]] = data[1:]
        elif key == "FORMAT":
            data = value.split(",")
            self.format[data[0]] = data[1:]
        elif key=="fileformat":
            self.fileformat = value

    def next( self ):
        
        data = self.line[:-1].split("\t")
        self.line = self.infile.readline()
        if not self.line: raise StopIteration
        return VCFEntry(data, self.samples )
            
if __name__ == "__main__":
    
    inf = VCFFile(sys.stdin)
    
    for x in inf:
        print str(x)



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
check_db.py - do a sanity check for a collection of linked tables
=================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

field-in-table:  count number of entries of field in tables
fields-in-table: count number on non-null values in all fields of a table
tables-on-field: count tables

Usage
-----

Example::

   python check_db.py --help

Type::

   python check_db.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import time
import optparse
import tempfile
import subprocess
import types
import numpy
import CGAT.Experiment as E

def output( counts, row_names, col_names, options ):

    outfile = open(options.output_filename_pattern % "counts", "w" )
    outfile.write ("\t%s\n" % "\t".join( col_names ) )
    for x, head in enumerate(row_names):
        outfile.write("%s\t%s\n" % (head, "\t".join( [ "%i" % i for i in counts[x,] ] )) )
    outfile.close()

    percents = numpy.zeros( (len(row_names), len(col_names)), numpy.float )
    for x in range(len(row_names)):
        m = max(counts[x,:])
        if m != 0:
            for y in range(len(col_names)):
                percents[x,y] = 100.0 * float(counts[x,y]) / m

    def __output( filename, matrix, pattern ):

        outfile = open(filename, "w" )
        outfile.write ("head\t%s\n" % "\t".join( col_names ) )
        for x, head in enumerate(row_names):
            outfile.write("%s\t%s\n" % (head, "\t".join( [ pattern % i for i in matrix[x,] ] )) )
        outfile.close()

    __output(options.output_filename_pattern % "counts", counts, "%i" )
    __output(options.output_filename_pattern % "percents", percents, "%5.2f" )


def doColumnInTables( dbhandle, error, options ):

    if options.backend == "pg":
        pass
    elif options.backend == "sqlite":
        cc = dbhandle.cursor()
        cc.execute("select tbl_name, sql from sqlite_master where type='table'")
        # tables returned are unicode, convert to string and filter
        # for presence of the join column 
        tag = " %s " % options.column
        tables = [ str(x[0]) for x in cc.fetchall() if tag in x[1] ]
        cc.close()

    E.debug( "tablenames from sql=%s" % str(tables) )

    if options.exclude_pattern:
        rx = re.compile( options.exclude_pattern )
        tables = [ x for x in tables if not re.search( rx, x ) ]
        E.debug( "tablenames after filtering=%s" % str(tables) )

    reg_head = re.compile(options.regex_head)
    reg_tail = re.compile(options.regex_tail)
    heads,tails = set(), set()
    for x in tables:
        try: heads.add( reg_head.search(x).groups()[0] )
        except AttributeError: pass
        try: tails.add( reg_tail.search(x).groups()[0] )
        except AttributeError: pass

    heads, tails = list(heads), list(tails)
    heads.sort()
    tails.sort()
    counts = numpy.zeros( (len(heads), len(tails)), numpy.int )

    map_head2index = dict( [(x[1],x[0]) for x in enumerate(heads)  ] )
    map_tail2index = dict( [(x[1],x[0]) for x in enumerate(tails)  ] )

    E.debug( "map_head2index=%s" % str(map_head2index) )
    E.debug( "map_tail2index=%s" % str(map_tail2index) )

    statement = "SELECT COUNT( DISTINCT %s) FROM %%s" % options.column

    E.info( "counting %s tables " % len(tables) )

    for table in tables:

        try: x = map_head2index[ reg_head.search(table).groups()[0] ]
        except AttributeError: continue
        try: y = map_tail2index[ reg_tail.search(table).groups()[0] ]
        except AttributeError: continue

        cc = dbhandle.cursor()
        cc.execute( statement % table )
        count = cc.fetchone()[0]
        cc.close()

        E.debug( "%s: %i: %s" % (table, count, statement % table) )
        counts[x,y]=count

    return counts, heads, tails

def doColumnsInTable( dbhandle, error, options ):

    rx = re.compile( options.table )

    if options.backend == "pg":
        pass
    elif options.backend == "sqlite":
        cc = dbhandle.cursor()
        cc.execute("select tbl_name FROM sqlite_master where type='table'")
        # tables returned are unicode, convert to string and filter
        # for presence of the join column 
        tables = [ str(x[0]) for x in cc.fetchall() if rx.search( x[0] ) ]
        cc.close()

        columns = set()
        for table in tables:
            cc = dbhandle.cursor()
            columns.update( set( [str(x[1]) for x in cc.execute("PRAGMA table_info( %s )" % table).fetchall()] ) )
            cc.close()

    tables, columns = list(tables), list(columns)
    tables.sort()
    columns.sort()

    E.debug( "tables=%s" % str(tables) )
    E.debug( "columns=%s" % str(columns) )

    counts = numpy.zeros( (len(tables), len(columns)), numpy.int )

    statement = "SELECT COUNT( %s) FROM %s WHERE %s IS NOT NULL" 

    E.info( "counting %i columns in %i tables " % (len(columns), len(tables) ))
    
    for x, table in enumerate(tables):

        for y, column in enumerate(columns):
            cc = dbhandle.cursor()
            try:
                cc.execute( statement % (column,table,column) )
                count = cc.fetchone()[0]
                cc.close()
            except error, msg:
                continue
            counts[x,y]=count

    new_tables = []
    rx = re.compile( options.regex_head )
    for table in tables:
        try:
            new_tables.append( rx.search( table ).groups()[0])
        except ValueError:
            new_tables.append( table )
            
    tables = new_tables

    return counts, tables, columns 

def main():

    parser = E.OptionParser( version = "%prog version: $Id: check_db.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-d", "--database", dest="database", type="string",
                      help="database name [default=%default]." )

    parser.add_option("-t", "--table", dest="table", type="string",
                      help="table name [default=%default]." )

    parser.add_option("-c", "--column", dest="column", type="string",
                      help="column name [default=%default]." )

    parser.add_option( "--regex-head", dest="regex_head", type="string",
                      help="regular expression to extract the head of a table, which will be the rows [default=%default]." )

    parser.add_option( "--regex-tail", dest="regex_tail", type="string",
                      help="regular expression to extract the tail of a table, which be the columns [default=%default]." )

    parser.add_option("-b", "--backend", dest="backend", type="choice",
                      choices=("pg", "sqlite"),
                      help="database backend to choose [default=%default]." )

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string" ,
                       help="pattern for output filenames [%default].")

    parser.add_option( "-e", "--exclude", dest="exclude_pattern", type="string" ,
                       help="pattern to exclude tables from the analysis [%default].")

    parser.add_option( "-m", "--method", dest="method", type="choice",
                      choices=("column-in-tables", "columns-in-table", "tables-on-column" ),
                      help="analysis to perform [default=%default]." )

    parser.set_defaults(
        database = "csvdb",
        backend="sqlite",
        column = None,
        table = None,
        regex_head = "^([^_]+)_",
        regex_tail = "_(.*)",
        output_filename_pattern = "%s",
        method = "column-in-tables",
        exclude_pattern = None,
        )

    (options, args) = E.Start( parser, add_psql_options = True )

    if options.backend == "pg":
        import pgdb
        dbhandle = pgdb.connect( options.psql_connection )
        error = pgdb.DatabaseError
        options.null = "NULL"
        options.string_value = "'%s'"
        if options.insert_quick:
            raise ValueError("quick import not implemented.")

    elif options.backend == "sqlite":
        import sqlite3
        dbhandle = sqlite3.connect( options.database )
        error = sqlite3.OperationalError
        options.insert_many = True  # False
        options.null = None # "NULL" 
        options.string_value = "%s" # "'%s'"

    if options.method == "column-in-tables":
        if not options.column:
            raise ValueError( "please supply a column name or pattern" )

        counts, row_names, column_names = doColumnInTables( dbhandle, error, options )

    elif options.method == "columns-in-table":
        if not options.table:
            raise ValueError( "please supply a table name or pattern" )
        counts, row_names, column_names = doColumnsInTable( dbhandle, error, options )

    output( counts, row_names, column_names, options )

    E.Stop()

if __name__ == "__main__":
    sys.exit(main())



#! /bin/env python
################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
run.py - wrapper around command line
====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script is a generic wrapper for tools with a command line interface.
The wrapper records execution times and logs successfull completion
of the command.

Usage
-----

Example::

   python run.py ls

Type::

   python run.py --help

for command line help.

Documentation
-------------

Code
----

'''
USAGE="""python run.py [OPTIONS] 

generic wrapper around a command-line. 

The wrapper records where a command was executed and how long
it took.
"""

import os
import sys
import re
import string
import optparse
import time
import glob
import subprocess

import CGAT.Experiment as E

if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: run.py 2782 2009-09-10 11:40:29Z andreas $", 
                                    usage = globals()["__doc__"] )

    ## stop parsing options at the first argument
    parser.disable_interspersed_args()

    (options, args) = E.Start( parser, 
                               add_pipe_options = True )
    
    if len(args) > 0:
        
        cmd = args[0]
        if len(args) > 1:
            cmd += " '" + "' '".join(args[1:]) + "'" 
    
        s = subprocess.Popen( cmd,
                              shell = True,
                              cwd = os.getcwd(),
                              close_fds = True)                              

        (out, err) = s.communicate()
        returncode = s.returncode
    else:
        returncode = 0

    E.Stop()

    sys.exit( returncode )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
plot_data.py - create scatter plot from tabular data
====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This scripts creates a scatter plot from tabular data.

Reads data in tabular format with observations in rows in
different samples in columns. The first column provides the
bins.

Reads data from stdin unless infile is given.

Usage
-----

Example::

   python plot_data.py --help

Type::

   python plot_data.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse

import CGAT.Experiment as E
import CGAT.Histogram as Histogram
import CGAT.MatlabTools as MatlabTools
import numpy
import numpy.ma as MA

def readTable( file,
               separator = "\t",
               numeric_type = numpy.float,
               take = "all",
               has_headers = True,
               truncate = None,
               missing_value = -99999,
               color = None,
               ):
    """read a matrix.

    There probably is a routine for this in Numpy, which I haven't found yet.
    Uses the MA module to deal with missing values.
    """

    lines = filter( lambda x: x[0] != "#", file.readlines())

    if len(lines) == 0:
        raise ValueError( "no data" )

    if take == "all":
        num_cols = len(string.split(lines[0][:-1], "\t"))
        take = range( 0, num_cols)
    else:
        num_cols = len(take)
        
    colors = None
    if color != None:
        take = [ x for x in take if x != color ]
        num_cols = len(take)
        colors = []

    if has_headers:
        headers = lines[0][:-1].split("\t")
        headers = map( lambda x: headers[x], take )        
        del lines[0]
    else:
        headers = None

    num_rows = len(lines)

    matrix = numpy.zeros( (num_rows, num_cols), numeric_type )

    nrow = 0
    if truncate:
        min_val, max_val = truncate
        
    for l in lines:
        data = l[:-1].split("\t")

        if color != None: 
            colors.append( float(data[color]) )

        try:
            data = map( lambda x: data[x], take)
        except IndexError:
            continue

        ## try conversion. Unparseable fields set to missing_value
        for x in range(len(data)):
            try:
                data[x] = float(data[x])
            except ValueError:
                data[x] = missing_value

        if truncate:
            if data[0] < min_val or data[0] > max_val: continue
            
        matrix[nrow] = data
        nrow += 1

    if truncate != None:
        # truncate matrix
        matrix = matrix[ 0:nrow+1, 0:num_cols]

    return MA.masked_values( matrix, missing_value ), headers, colors

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: plot_data.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-f", "--file", dest="filenames", type="string",
                      help="files[s] to take data from,stdin = -."  )
    parser.add_option("-l", "--legend", dest="legend", type="string",
                      help="legend for plot. If not given, the legend is build from the column titles [default=%Default]."  )
    parser.add_option("--no-legend", dest="no_legend", action="store_true",
                      help="do not output a legend [default=%Default]"  )
    parser.add_option("-t", "--title", dest="title", type="string",
                      help="title for plot."  )
    parser.add_option("-p", "--hardcopy", dest="hardcopy", type="string",
                      help="filename for hardcopy.", metavar = "FILE"  )
    parser.add_option("", "--xrange", dest="xrange", type="string",
                      help="xrange."  )
    parser.add_option("", "--yrange", dest="yrange", type="string",
                      help="yrange."  )
    parser.add_option("-o", "--logscale", dest="logscale", type="string",
                      help="log-transform one or both axes [default=%Default]."  )
    parser.add_option("-x", "--xtitle", dest="xtitle", type="string",
                      help="title for x axis"  )
    parser.add_option("-y", "--ytitle", dest="ytitle", type="string",
                      help="title for y axis"  )
    parser.add_option("-n", "--normalize", dest="normalize", action = "store_true",
                      help="normalize histograms"  )
    parser.add_option("", "--legend-location", dest="legend_location", type="string",
                      help="location of legend"  )
    parser.add_option("", "--backend", dest="backend", type="string",
                      help="backend to use [Agg|SVG|PS]"  )
    parser.add_option("", "--symbols", dest="symbols", type="string",
                      help="symbols to use for each histogram [steps|...]."  )
    parser.add_option("", "--dump", dest="dump", action="store_true",
                      help="dump data for debug purposes."  )
    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to use for plotting [default=%Default].")
    parser.add_option( "--color", "--colour", dest="color", type="int",
                      help="column with field to use for coloring [default=%Default].")
    parser.add_option("--transpose", dest="transpose", action="store_true",
                      help="transpose values (swop x and y axis).")
    parser.add_option("--truncate", dest="truncate", action="store_true",
                      help="truncate histogram before plotting to xrange.")
    parser.add_option("--layout", dest="layout", type="string",
                      help="layout.")
    parser.add_option("--scatter", dest="scatter", action="store_true",
                      help="do a scatter plot.")
    parser.add_option("--point-size", dest="point_size", type="int",
                      help="point-size.")
    parser.add_option("--add-function", dest="function", type="string",
                      help="add a function to the plot." )
    parser.add_option("--add-error-bars", dest="error_bars", type="choice",
                      choices=("interleaved", "block"),
                      help="add error bars." )
    parser.add_option( "--no-titles", dest ="has_headers", action="store_false",
                       help="no column titles given." )
    parser.add_option( "--palette", dest="palette", type="choice",
                       choices=("rainbow", "gray", "blue-white-red",
                                "autumn", "bone", "cool", "copper", "flag", "gray", "hot", "hsv", "jet", "pink", "prism",
                                "spring", "summer", "winter", "spectral",
                                "RdBu", "RdGy", "BrBG", "BuGn", "Blues", "Greens", "Reds", "Oranges", "Greys" ),
                       help="colour palette [default=%Default]")
    parser.add_option( "--reverse-palette", dest="reverse_palette", action="store_true",
                       help="reverse colour palette [default=%Default].")

    parser.set_defaults(
        filenames = "-",
        legend = None,
        title = None,
        hardcopy = None,
        logscale = None,
        xtitle = None,
        ytitle = None,
        xrange = None,
        yrange = None,        
        normalize = None,
        columns = "all",
        legend_location = "upper right",
        backend = "Agg",
        symbols = "gD,bh,r+,c+,m+,y+,ko,g^,b<,r>,cD,mh",
        dump = False,
        truncate = False,
        layout = "1,1",
        scatter = False,
        point_size=0,
        transpose = False,
        function = None,
        error_bars = None,
        has_headers = True,
        color = None,
        palette = None,
        reverse_palette = False,
        no_legend = False,
        )

    (options, args) = E.Start( parser )

    options.symbols=options.symbols.split(",")

    # import matplotlib/pylab. Has to be done here
    # for batch scripts without GUI.
    import matplotlib
    if options.hardcopy: matplotlib.use( "cairo" )
    import pylab

    plot_kwargs = {}
    
    if options.xrange: options.xrange = map(float, options.xrange.split(","))
    if options.yrange: options.yrange = map(float, options.yrange.split(","))
    if options.legend: options.legend = options.legend.split(",")  
    if options.layout: options.layout = tuple(map(int, options.layout.split(",")))
    if options.color: 
        options.color -= 1
        plot_kwargs["edgecolor"] = 'none'
    if options.columns != "all": options.columns = map(lambda x: int(x)-1, options.columns.split(","))

    if options.palette:
        if options.palette == "gray":
            color_scheme = colors_gray
        else:
            if options.reverse_palette:
                color_scheme = eval( "pylab.cm.%s_r" % options.palette)                    
            else:
                color_scheme = eval( "pylab.cm.%s" % options.palette)
        plot_kwargs["cmap"] = color_scheme
    else:
        color_scheme = None
    
    options.filenames = options.filenames.split(",")
    
    if len(args) > 0:
        options.filenames = args

    plotno = 0
    for filename in options.filenames:
        if filename == "-":
            infile = sys.stdin
        else:
            infile = open(filename)

        try:
            data, legend, colors = readTable( infile,
                                              numeric_type=numpy.float32,
                                              take=options.columns,
                                              has_headers = options.has_headers,
                                              truncate= options.xrange,
                                              color = options.color )
        except ValueError, msg:
            E.warn( "parsing error in %s: %s" % (filename, msg) )
            continue

        if filename != "-":
            infile.close()

        if not options.has_headers:
            legend = ["col%i" % i for i in range( 0, len(data[0]) ) ]

        if options.legend: legend = options.legend
        
        plotno += 1
        pylab.subplot( "".join(map(str, options.layout + (plotno,))))

        if options.title:
            pylab.title( options.title )

        if options.xtitle:
            xlab = options.xtitle      
        else:
            xlab = legend[0] 
    
        if options.ytitle:
            ylab = options.ytitle
        else:
            ylab = None

        if options.transpose:
            xlab, ylab = ylab, xlab

        if xlab: pylab.xlabel( xlab )
        if ylab: pylab.ylabel( ylab )
        
        if options.normalize:
            m = [0] * len(legend)
            for d in data:
                for x in range(1, len(legend)):
                    m[x] = max(m[x], d[x])
            for d in data:
                for x in range(1, len(legend)):
                    if m[x] > 0:
                        d[x] = d[x] / m[x]

        if options.dump:
            for d in data:
                print d

        lines = []
        
        for x in range(1, len(legend)):
            s = options.symbols[x % len(options.symbols)]

            if not options.transpose:
                first, second = 0, x
            else:
                first, second = x, 0

            yvals = data[:,second]
            xvals = MA.masked_array(data[:,first], MA.getmask(yvals))
            
            xvals = xvals.compressed()
            yvals = yvals.compressed()            

            if options.scatter:
                if options.point_size:
                    lines.append (pylab.scatter( xvals, yvals, options.point_size, c = colors, **plot_kwargs ))
                else:
                    # lines.append ( pylab.scatter( xvals, yvals, c = colors ) ) # , vmin=min(colors),vmax=max(colors), facetted=False))
                    lines.append( pylab.scatter( xvals, yvals, c = colors, vmin=min(colors),vmax=max(colors), **plot_kwargs ) )
            else:
                if options.error_bars:
                    lines.append(pylab.errorbar( xvals, yvals,
                                                 yerr = None,
                                                 fmt=s ) )
                else:
                    assert len(xvals) == len(yvals), "unequal number of values: %i != %i" % (len(xvals),len(yvals))
                    lines.append(pylab.plot( xvals, yvals, s ) )

        if not options.no_legend:
            pylab.legend( lines, legend[1:], options.legend_location)

        if options.logscale:
            if "x" in options.logscale:
                pylab.gca().set_xscale('log')
            if "y" in options.logscale:
                pylab.gca().set_yscale('log')

        if options.xrange:
            pylab.xlim( options.xrange )
            
        if options.yrange:
            pylab.ylim( options.yrange )

        if options.color != None:
            pylab.colorbar()

    if options.function:
        xstart, xend = pylab.gca().get_xlim()
        increment = (xend - xstart) / 100.0
        exec("f = lambda x: %s" % options.function )
        xvals, yvals = [], []
        for x in range(0,100):
            xvals.append( xstart )
            yvals.append( f(xstart) )
            xstart += increment
        xvals.append( xstart )
        yvals.append( f(xstart) )
        
        pylab.plot( xvals, yvals )
            
    if options.hardcopy:
        pylab.savefig( os.path.expanduser(options.hardcopy) )
    else:
        pylab.show()

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2011 David Sims
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
====================================================
Add a number of random reads to an aligned bam file
====================================================

:Author: David Sims
:Release: $Id$
:Date: |today|
:Tags: Python

This program takes two input file: a Samtools indexed genome file and a bam file to add reads to.
It also takes several input parameters: number of reads to be added, the read length, the insert size and the insert size standard deviation
Currently the program only adds paired end reads, with both reads the same length
The default insert size is 250, and insert size standard diviation is 20.
The default read length is 50 and the number of reads to be added is 10,000.


Usage
=====

add_random_reads_to_bam.py -g <genome.fa> -b <input.bam>-o <output.bam> -i insert size -s insert size sd -r number of reads -l read length


Code
====

"""

# load modules

import logging as L
import CGAT.Experiment as E
import CGAT.Pipeline as P
import sys
import os
import re
import shutil
import itertools
import math
import glob
import time
import gzip
import collections
import random
import optparse
import CGAT.GTF as GTF
import CGAT.IOTools as IOTools
import CGAT.IndexedFasta as IndexedFasta

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: snp2table.py 2861 2010-02-23 17:36:32Z davids $", usage = globals()["__doc__"] )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with Samtools indexed genome [default=%default]."  )
    parser.add_option("-b", "--bam-file", dest="bam_file", type="string",
                      help="filename of bam to add reads to [default=%default]."  )
    parser.add_option("-i", "--insert-size", dest="isize", type="string",
                      help="Insert size [default=%default]."  )
    parser.add_option("-s", "--insert-sd", dest="isd", type="string",
                      help="Insert size standard deviation [default=%default]."  )
    parser.add_option("-r", "--nreads", dest="nreads", type="string",
                      help="Number of random reads to add [default=%default]."  )
    parser.add_option("-l", "--readlength", dest="readlength", type="string",
                      help="length of reads to generate [default=%default]."  )
    parser.add_option("-o", "--output", dest="output_file", type="string",
                      help="output filename  [default=%default]."  )

    parser.set_defaults(
        genome_file = None,
        bam_file = None,
        isize=250,
        isd=20,
        nreads=10000,
        readlength=50,
        output_file = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # Generate random reads and add to bam
    track = P.snip(os.path.basename(options.bam_file),".bam")
    readlen = options.readlength
    isize = options.isize
    isd = options.isd
    nreads = options.nreads
    genome = options.genome_file
    bam = options.bam_file
    out = options.output_file
    statement = '''java -jar -Xmx2048m /ifs/apps/bio/simseq-72ce499/SimSeq.jar 
                       -1 %(readlen)s -2 %(readlen)s \
                       --error  /ifs/apps/bio/simseq-72ce499/examples/hiseq_mito_default_bwa_mapping_mq10_1.txt \
                       --error2 /ifs/apps/bio/simseq-72ce499/examples/hiseq_mito_default_bwa_mapping_mq10_2.txt \
                       --insert_size %(isize)s \
                       --insert_stdev %(isd)s \
                       --read_number %(nreads)s \
                       --read_prefix simseq_ \
                       --reference %(genome)s \
                       --duplicate_probability 0.0 \
                       --out simseq.sam > simseq.log; ''' % locals()
    P.run()
    statement = '''samtools view -bS -t %(genome)s.fai -o simseq.bam simseq.sam; 
                   samtools sort simseq.bam simseq.srt;
                   samtools sort %(bam)s %(track)s.srt;
                   samtools merge %(out)s %(track)s.srt.bam simseq.srt.bam;
                   samtools index %(out)s;'''
    P.run()

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Database.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, time, string, traceback, re
import MySQLdb
import _mysql           # necessary for exception handling

WARNINGS = 1                    # print out warnings?

class Database:
    dbhandle = None

    socket = None
    host = None
    user = None
    passwd = None
    port = None
    dbname = None

    ## perform retries if you lost connection
    ## by getting a new connection.
    mRetryLostConnection = 1

    
    mRetryTimeIntervall  = 60


    def __init__(self):
        pass

    def AddOptions( self, parser ):
        """add connection options to a command line parser.
        """
        
        parser.add_option("-D", "--Database", dest="database", type="string",
                          help="database to connect to.")
        
        parser.add_option("-H", "--Host", dest="host", type="string",
                          help="host to connect to.")

        parser.add_option("-U", "--Username", dest="user", type="string",
                          help="username in database.")

        parser.add_option("-W", "--Password", dest="passwd", type="string",
                          help="password for database.")

        parser.add_option("-S", "--Socket", dest="socket", type="string",
                          help="socket of database connection.")

        parser.add_option("-P", "--Port", dest="port", type="string",
                          help="port to use.")

        parser.set_defaults(
            host = None,
            database = "",
            user = None,
            passwd = "",
            port = None, 
            socket = "/tmp/mysql.sock" )
            
    def Connect( self, options ):

	try:
            if options.port:
                self.dbhandle = MySQLdb.connect(host        = options.host,
                                                user        = options.user,
                                                passwd      = options.password,
                                                db          = options.database,
                                                port        = options.port)
            else:
                self.dbhandle = MySQLdb.connect(host        = options.host,
                                                user        = options.user,
                                                passwd      = options.password,
                                                db          = options.database,
                                                unix_socket = options.socket )
	    return self.dbhandle
	except _mysql.OperationalError, detail:
            print "------------------------"
	    print "MySQL raised error %s: %s!" % detail.args
            traceback.print_stack()
            print "------------------------"
	    return 0

    def Close( self ):
        """close connection. No further activity possible.
        """
        self.dbhandle.close()
	    
    def Execute( self, statement ):
        """execute a mysql statement and return a cursor object.

        Die, if there is an error.
        """
	try:
	    c = self.dbhandle.cursor()
	    c.execute( statement )
        except _mysql.ProgrammingError, detail:
            
            print """# mysql raised programming error %s: %s!
# %s""" % (detail.args[0], detail.args[1], statement)
            traceback.print_stack(file=sys.stdout)                        
            sys.exit(1)
            
	except _mysql.OperationalError, detail:
            print """# mysql raised operational error %s: %s!
# %s""" % (detail.args[0], detail.args[1], statement)
            traceback.print_stack(file=sys.stdout)                        
            
            if detail.args[0] == "2013" and self.mRetryLostConnection:
            
                while 1:
            
                    sys.stderr.write( "retrying to establish connection\n" )
                    sys.stderr.flush()
                
                    time.sleep(self.mRetryTimeIntervall)
                    if self.Connect( host = self.host,
                                     user = self.user,
                                     passwd = self.passwd,
                                     dbname = self.dbname,
                                     port = self.port,
                                     socket = self.socket ):
                        break

                ## carefull: infinite loop!
                return self.Execute(statement)
            
            sys.exit(1)
                
	except _mysql.Warning, detail:
            if WARNINGS:
                print "------------------------"
                print "MySQL raised a warning: %s!" % detail.args
                traceback.print_stack(file=sys.stdout)
                print "Offending statement:"
                print statement
                print "------------------------"

        ## check, whether query was interrupted:
        errno = self.dbhandle.errno()
        if errno != 0:
            print """mysql error %i: %s!
%s""" % ( errno, self.dbhandle.error(), statement)
            traceback.print_stack(file=sys.stdout)
            sys.exit(1)
            
        return c

    def QuoteString( self, text ):
	return _mysql.escape_string(text)

    def GetDate(self):			# return date in format yyyy-mm-dd
	return time.strftime("%Y-%m-%d", time.localtime(time.time()))

    def Create( self ):
        if len(self.socket) == 0:
            self.socket = "/tmp/mysql.sock"             # use standard socket under unix

	try:
	    self.dbhandle = MySQLdb.connect(host        = self.host,
					    user        = self.user,
					    passwd      = self.passwd,
					    unix_socket = self.socket )
	except _mysql.OperationalError, detail:
	    print "MySQL raised error %s: %s!" % detail.args  
	    return 0

	c  = self.dbhandle.cursor()
	
	try:
	    c.execute('CREATE DATABASE %s' % self.dbname)
            return 1
	except _mysql.OperationalError, detail:
	    print "MySQL raised error %s: %s!" % detail.args
            return 0
    
    def Get_Tables( self, database = None):
        
        if database:
            statement = "SHOW TABLES FROM " + database
        else:
            statement = "SHOW TABLES"
            
        return map(lambda x: x[0], self.Execute(statement).fetchall())

    def DropTable( self, table_name):
        """drop table.
        """
        return self.Execute("DROP TABLE IF EXISTS %s" % table_name)

    def Exists( self, table_name ):
        """returns 1, if a table with name name exists."""
        # How can I locate an element in a list without iterating through it
        # myself? The code below looks ugly (exceptions, but this is the Python style?).

        r = string.split(table_name, ".")
        if len(r) > 1:
            database = r[0]
            table = r[1]
        else:
            database = None
            table = r[0]
        
        tables = self.Get_Tables(database)
        try:
            tables.index(table)
            return 1
        except (ValueError):
            return 0


    #--------------------------------------------------------------------------------------
    def GetDatabase( self ):
        """return current database name."""
        return self.dbname
    
    #--------------------------------------------------------------------------------------    
    def UseDatabase( self, db_name ):
        """switch database."""
        self.dbname = db_name
        return self.Execute( "USE %s " % db_name )

    #--------------------------------------------------------------------------------------
    def GetUser(self):
        return self.user
    #--------------------------------------------------------------------------------------
    def GetHost(self):
        return self.host
    #--------------------------------------------------------------------------------------
    def GetPort(self):
        return self.port
    #--------------------------------------------------------------------------------------
    def GetPassword(self):
        return self.passwd
    
    #------------------------------------------------------------------------------------------------------
    def SelectIntoOutfile( self, statement, filename ):
        """Redirect SELECT-statement INTO outfile on local disc. As this
        is not directly possible via mysqld, mysql -e > outfile is used.

        Note: password is not given here, it has to be set in .my.cnf,
        as otherwise it is visible via ps to all.

        options for mysql:
        -N: skip column headers
        """
        
        if os.system( "mysql -N -h%s -u%s -P%i -e '%s' > %s" % (self.host, self.user,
                                                                self.port,
                                                                statement, filename)):
            print "--> error causing statement: %s" % statement
        
    #------------------------------------------------------------------------------------------------------
    def LoadFile( self, filename, statement ):
        """load a file using statement.
        """
        
        os.chmod( filename, 0664)
        self.Execute( statement % filename )
        
        return 

def executewait( dbhandle, statement, error = None, retries = -1, wait=5):
    '''execute an sql ``statement``.

    If ``error`` is given, it is scanned for locking issues.

    Retry ``retries`` times if set to a positive number.
    A retry of ``0`` indicates no retry, a negative number retries
    infinitely. 

    The process waits ``wait`` seconds between each retry.

    Returns a cursor object.
    '''
    if error == None: error = Exception

    cc = dbhandle.cursor()    

    while 1:
        try:
            cc.execute( statement )
        except error, msg:
            if retries == 0:
                raise error, msg
            if not re.search("locked", str(msg)):
                raise error, msg
            time.sleep(wait)
            retries -= 1
            continue
        break
    return cc

def getColumnNames( dbhandle, table ):
    '''get column names of a table from a database.'''
    
    cc = executewait( dbhandle, "SELECT * FROM %s LIMIT 1" % table )
    return tuple([ x[0] for x in cc.description ])


def getTables( dbhandle ):
    
    cc = executewait( dbhandle, """select name from sqlite_master where type='table'""")
    return tuple([ x[0] for x in cc ])




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
analyze_codonbias_orthology.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python analyze_codonbias_orthology.py --help

Type::

   python analyze_codonbias_orthology.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import math

"""analyze codonbias of orthologs.

The input looks like:

schema1 gene1 cai1 schema2, gene2, cai2
"""

import CGAT.Experiment as E

def GetThreshold( dbhandle, schema ):
    """get dominant set threshold for schema."""

    statement = "SELECT cai FROM %s.codonbias WHERE is_selected = TRUE ORDER BY cai DESC";
    
    ## get thresholds
    cc = dbhandle.cursor()
    cc.execute(statement)
    r = cc.fetchall()
    cc.close()

    return threshold

##--------------------------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: analyze_codonbias_orthology.py 2781 2009-09-10 11:33:14Z andreas $")
    
    parser.add_option( "--filters", dest="filters", type="string",
                      help="Filters to use for filtering sequences [all|codon1|codon2|codon3|44]." )
    parser.add_option( "--fields", dest="fields", type="string",
                      help="Fields to output [aligned|nunaligned1|nunaligned2|identical|transitions|transversions|jc69|t92]." )

    parser.set_defaults(
        filename_map = None,
        gap_char = "-",
        )
    
    (options, args) = E.Start( parser, add_pipe_options = True,
                                        add_psql_options = True)

    data = map(lambda x: x[:-1].split("\t"), filter( lambda x: x[0] != "#", sys.stdin.readlines()))

    schema1 = data[0][0]
    schema2 = data[0][3]

    dbhandle = pgdb.connect( options.psql_connection )
    
    threshold1 = GetThreshold( dhhandle, schema1)
    threshold1 = GetThreshold( dhhandle, schema2)
    

    
    

    
    E.Stop()



#cython: embedsignature=True
cimport cython

###############################
# Could not make .pxd file to be found in gpipe/setup.py, so including it here:
# Fields to extension classed have been added to each class.
###############################
cdef extern from "string.h":
  ctypedef int size_t
  void *memcpy(void *dst,void *src,size_t len)
  void *memmove(void *dst,void *src,size_t len)
  void *memset(void *b,int c,size_t len)

cdef extern from "stdlib.h":
  void free(void *)
  void *malloc(size_t)
  void *calloc(size_t,size_t)
  void *realloc(void *,size_t)
  int c_abs "abs" (int)
  void qsort(void *base, size_t nmemb, size_t size,
             int (*compar)(void *,void *))

cdef extern from "stdio.h":
  ctypedef struct FILE:
    pass
  FILE *fopen(char *,char *)
  int fclose(FILE *)
  int sscanf(char *str,char *fmt,...)
  int sprintf(char *str,char *fmt,...)
  int fprintf(FILE *ifile,char *fmt,...)
  char *fgets(char *str,int size,FILE *ifile)

cdef extern from "string.h":
  int strcmp(char *s1, char *s2)
  int strncmp(char *s1,char *s2,size_t len)
  char *strcpy(char *dest,char *src)
  char *strdup(char *)
  char *strcat(char *,char *)

cdef extern from "intervaldb.h":
  ctypedef struct IntervalMap:
    int start
    int end
    int target_id
    int sublist

  ctypedef struct IntervalIndex:
    int start
    int end

  ctypedef struct SublistHeader:
    int start
    int len

  ctypedef struct SubheaderFile:
    pass
  
  ctypedef struct IntervalDBFile:
    int n
    int ntop
    int nlists
    int div
    int nii
    IntervalIndex *ii
    SublistHeader *subheader
    SubheaderFile subheader_file
    FILE *ifile_idb

  ctypedef struct IntervalIterator:
    pass

  ctypedef struct FilePtrRecord:
    FILE *ifile
    int left
    int right
    int ihead
    char *filename

  int imstart_qsort_cmp(void *void_a,void *void_b)
  int target_qsort_cmp(void *void_a,void *void_b)
  IntervalMap *read_intervals(int n,FILE *ifile) except NULL
  SublistHeader *build_nested_list(IntervalMap im[],int n,int *p_n,int *p_nlists) except NULL
  SublistHeader *build_nested_list_inplace(IntervalMap im[],int n,int *p_n,int *p_nlists) except NULL
  IntervalMap *interval_map_alloc(int n) except NULL
  IntervalIterator *interval_iterator_alloc() except NULL
  int free_interval_iterator(IntervalIterator *it)
  IntervalIterator *reset_interval_iterator(IntervalIterator *it)
  int find_intervals(IntervalIterator *it0,int start,int end,IntervalMap im[],int n,SublistHeader subheader[],int nlists,IntervalMap buf[],int nbuf,int *p_nreturn,IntervalIterator **it_return) except -1
  char *write_binary_files(IntervalMap im[],int n,int ntop,int div,SublistHeader *subheader,int nlists,char filestem[])
  IntervalDBFile *read_binary_files(char filestem[],char err_msg[],int subheader_nblock) except NULL
  int free_interval_dbfile(IntervalDBFile *db_file)
  int find_file_intervals(IntervalIterator *it0,int start,int end,IntervalIndex ii[],int nii,SublistHeader subheader[],int nlists,SubheaderFile *subheader_file,int ntop,int div,FILE *ifile,IntervalMap buf[],int nbuf,int *p_nreturn,IntervalIterator **it_return) except -1
  int write_padded_binary(IntervalMap im[],int n,int div,FILE *ifile)
  int read_imdiv(FILE *ifile,IntervalMap imdiv[],int div,int i_div,int ntop)
  int save_text_file(char filestem[],char basestem[],char err_msg[],FILE *ofile)
  int text_file_to_binaries(FILE *infile,char err_msg[])
  int C_int_max

###############################
### .pxd end
###############################

cdef class IntervalDBIterator:
  """Iterator over intervals from an NCL."""

  cdef IntervalIterator *it,*it_alloc
  cdef IntervalMap im_buf[1024]
  cdef int ihit,nhit,start,end
  cdef IntervalDB db

  def __cinit__(self,int start,int end,IntervalDB db not None):
    self.it=interval_iterator_alloc()
    self.it_alloc=self.it
    self.start=start
    self.end=end
    self.db=db

    # stop if interval is empty
    if self.start >= self.end: 
      raise IndexError( "invalid interval (%i,%i)" % (self.start, self.end) )

  def __iter__(self):
    return self 

  cdef int cnext(self): # C VERSION OF ITERATOR next METHOD RETURNS INDEX

    cdef int i
    if self.ihit>=self.nhit: # TRY TO GET ONE MORE BUFFER CHUNK OF HITS
      if self.it==NULL: # ITERATOR IS EXHAUSTED
        return -1
      find_intervals(self.it,self.start,self.end,self.db.im,self.db.ntop,
                     self.db.subheader,self.db.nlists,self.im_buf,1024,
                     &(self.nhit),&(self.it)) # GET NEXT BUFFER CHUNK
      self.ihit=0 # START ITERATING FROM START OF BUFFER
    if self.ihit<self.nhit: # RETURN NEXT ITEM FROM BUFFER
      i=self.ihit
      self.ihit = self.ihit+1 # ADVANCE THE BUFFER COUNTER
      return i
    else: # BUFFER WAS EMPTY, NO HITS TO ITERATE OVER...
      return -1

  # PYTHON VERSION OF next RETURNS HIT AS A TUPLE
  def __next__(self): # PYREX USES THIS NON-STANDARD NAME INSTEAD OF next()!!!
    cdef int i
    i=self.cnext()
    if i>=0:
      return (self.im_buf[i].start,self.im_buf[i].end,self.im_buf[i].target_id )
    else:
      raise StopIteration

  def __dealloc__(self):
    'remember: dealloc cannot call other methods!'
    free_interval_iterator(self.it_alloc)


cdef class IntervalDB:
  """a NCL database in memory.

  The initial database is empty. Call any of the ``from``
  methods to populate the database.
  """
  cdef int n
  cdef int ntop
  cdef int nlists
  cdef IntervalMap *im
  cdef SublistHeader *subheader


  def __cinit__(self,**kwargs):
    self.n = 0

  def fromsortedfile( self, filename, nsize, **kwargs):
    """build database from *nsize* tuples in *filename*.

    Each line in the file should contain three whitespace separated integers
    with ``start``, ``end`` and ``index`` of an interval.

    The intervals have to be sorted.
    """
    cdef FILE *ifile
    cdef int i
    assert nsize > 0, "number of tuples must be larger than 0"
    self.n=nsize

    ifile=fopen(filename,"r")
    if ifile:
      self.im=read_intervals(self.n,ifile)
      fclose(ifile)
      if self.im!=NULL:
        self.runBuildMethod(**kwargs)
    else:
      raise IOError('could not open file %s' % filename)

  def fromlist(self, l, **kwargs):
    '''build from data in l.

    The data should be given as list of tuple of tuples
    ``(start,end,id)``.

    see :meth:runBuildMethod for *kwargs*.
    '''
    cdef int i
    self.close() # DUMP OUR EXISTING MEMORY 
    self.n=len(l)
    self.im=interval_map_alloc(self.n)
    if self.im==NULL:
      raise MemoryError('unable to allocate IntervalMap[%d]' % self.n)
    i=0
    for t in l:
      self.im[i].start=t[0]
      self.im[i].end=t[1]
      self.im[i].target_id=t[2]
      self.im[i].sublist= -1
      i=i+1
    self.runBuildMethod(**kwargs)

  def runBuildMethod(self, buildInPlace=True):
    '''build either in-place if *buildInPlace == True* or using older build method

    TODO: this method currently does not check for invalid intervals.
    '''
    if buildInPlace:
      self.subheader=build_nested_list_inplace(self.im,self.n,&(self.ntop),&(self.nlists))
    else:
      self.subheader=build_nested_list(self.im,self.n,&(self.ntop),&(self.nlists))

  def fromunsortedfile(self, filename,int n,**kwargs):
    """build database from *nsize* tuples in *filename*.

    The intervals need not be sorted.
    """
    cdef FILE *ifile
    cdef int i
    cdef IntervalMap *im_new
    self.close()
    ifile=fopen(filename,'r')
    if ifile==NULL:
      raise IOError('unable to open '+filename)
    im_new=interval_map_alloc(n)
    if im_new==NULL:
      raise MemoryError('unable to allocate IntervalMap[%d]' % n)
    i=read_imdiv(ifile,im_new,n,0,n)
    fclose(ifile)
    if i!=n:
      raise IOError('IntervalMap file corrupted?')
    self.n=n
    self.im=im_new
    self.runBuildMethod(**kwargs)

  def find_overlap(self,int start,int end):
    """find intervals in database overlapping with *start* and *end*.
    
    returns an :class:`ncl.IntervalDBIterator`
    """
    self.check_nonempty() # RAISE EXCEPTION IF NO DATA
    return IntervalDBIterator(start,end,self)

  def find_overlap_list(self,int start,int end):
    """return list of intervals in database overlapping with *start* and *end*.
    """
    cdef int i,nhit
    cdef IntervalIterator *it,*it_alloc
    cdef IntervalMap im_buf[1024]
    self.check_nonempty() # RAISE EXCEPTION IF NO DATA
    it=interval_iterator_alloc()
    it_alloc=it
    l=[] # LIST OF RESULTS TO HAND BACK
    while it:
      find_intervals(it,start,end,self.im,self.ntop,
                     self.subheader,self.nlists,im_buf,1024,
                     &(nhit),&(it)) # GET NEXT BUFFER CHUNK
      for i from 0 <= i < nhit:
        l.append((im_buf[i].start,im_buf[i].end,im_buf[i].target_id))
    free_interval_iterator(it_alloc)
    return l
        
  def check_nonempty(self):
    """return True if the database is empty."""
    if self.im:
      return True
    else:
      msg='empty IntervalDB, not searchable!'
      raise IndexError(msg)

  def write_binaries(self,filestem,div=256):
    """commit database to filesystem using *filestem* as 
    root name of files.

    *div* refers to the block size.
    """
    cdef char *err_msg
    err_msg=write_binary_files(self.im,self.n,self.ntop,div,
                               self.subheader,self.nlists,filestem)
    if err_msg:
      raise IOError(err_msg)

  def __dealloc__(self):
    'remember: dealloc cannot call other methods!'
    if self.subheader:
      free(self.subheader)
    if self.im:
      free(self.im)
    
  def close(self):
    if self.subheader:
      free(self.subheader)
    if self.im:

      free(self.im)
    self.subheader=NULL
    self.im=NULL

    return None

cdef class IntervalFileDB:
  """a NCL database on the filesystem identified by *filestem*.

  NCL disk-based databases can only be opened for reading.
  create the database by building it in memory using :class:`ncl.IntervalDB` and then saving
  it to disk.
  """
  cdef IntervalDBFile *db

  def __cinit__(self,filestem=None):
    if filestem is not None:
      self.open(filestem)

  def open(self,filestem):
    cdef char err_msg[1024]
    self.db=read_binary_files(filestem,err_msg,1024)
    if self.db==NULL:
      raise IOError(err_msg)

  def find_overlap(self,int start,int end):
    """find intervals in database overlapping with *start* and *end*.
    
    returns an :class:`ncl.IntervalDBIterator`
    """
    self.check_nonempty() # RAISE EXCEPTION IF NO DATA
    return IntervalFileDBIterator(start,end,self)

  def find_overlap_list(self,int start,int end):
    """return list of intervals in database overlapping with *start* and *end*.
    """
    cdef int i,nhit
    cdef IntervalIterator *it,*it_alloc
    cdef IntervalMap im_buf[1024]
    self.check_nonempty() # RAISE EXCEPTION IF NO DATA
    it=interval_iterator_alloc()
    it_alloc=it
    l=[] # LIST OF RESULTS TO HAND BACK
    while it:
      find_file_intervals(it,start,end,self.db[0].ii,self.db[0].nii,
                          self.db[0].subheader,self.db[0].nlists,
                          &(self.db[0].subheader_file),
                          self.db[0].ntop,self.db[0].div,
                          self.db[0].ifile_idb,im_buf,1024,
                          &(nhit),&(it)) # GET NEXT BUFFER CHUNK
      for i from 0 <= i < nhit:
        l.append((im_buf[i].start,im_buf[i].end,im_buf[i].target_id))
    free_interval_iterator(it_alloc)
    return l

  def check_nonempty(self):
    if self.db==NULL:
      raise IndexError('empty IntervalFileDB, not searchable!')

  def close(self):
    if self.db:
      free_interval_dbfile(self.db)
    self.db=NULL

  def __dealloc__(self):
    'remember: dealloc cannot call other methods!'
    if self.db:
      free_interval_dbfile(self.db)


cdef class IntervalFileDBIterator:
  """disk based intervalDB."""

  cdef IntervalIterator *it,*it_alloc
  cdef IntervalMap *im_buf
  cdef int ihit,nhit,start,end,nbuf
  cdef IntervalFileDB db
  cdef IntervalDB idb

  cdef int restart(self,int start,int end,IntervalFileDB db) except -2
  cdef int reset(self) except -2
  cdef int cnext(self,int *pkeep)
  cdef int extend(self,int ikeep)
  cdef int saveInterval(self,int start,int end,int target_id)
  cdef int nextBlock(self,int *pkeep) except -2
  cdef IntervalMap *getIntervalMap(self)
  cdef int loadAll(self) except -1
  cdef int copy(self,IntervalFileDBIterator src)

  def __cinit__(self,
              int start,int end,
              IntervalFileDB db=None,
              int nbuffer=1024,rawIvals=None):
    cdef int i

    if start >= end: 
      raise IndexError( "invalid interval (%i,%i)" % (start, end) )

    ns = None
    self.it=interval_iterator_alloc()
    self.it_alloc=self.it
    self.start=start
    self.end=end
    self.db=db
    if ns is not None:
      if ns.idb is not None:
        self.idb=ns.idb
      elif ns.db is None:
        ns.forceLoad()
      self.db=ns.db
    if rawIvals is not None and len(rawIvals)>nbuffer:
      nbuffer=len(rawIvals)
    self.im_buf=interval_map_alloc(nbuffer)
    self.nbuf=nbuffer
    if rawIvals is not None:
      i=0
      for ival in rawIvals:
        self.im_buf[i].start=ival[0] # SAVE INTERVAL INFO
        self.im_buf[i].end=ival[1]
        self.im_buf[i].target_id=ival[2]
        i=i+1
      self.nhit=i # TOTAL NUMBER OF INTERVALS STORED

  cdef int restart(self,int start,int end,IntervalFileDB db) except -2:
    'reuse this iterator for another search without reallocing memory'
    self.nhit=0 # FLUSH ANY EXISTING DATA
    self.start=start
    self.end=end
    self.db=db
    self.it=self.it_alloc # REUSE OUR CURRENT ITERATOR
    reset_interval_iterator(self.it) # RESET IT FOR REUSE
    return 0

  cdef int reset(self) except -2:
    'flush the buffer so we can reuse this iterator'
    self.nhit=0
    return 0

  cdef int extend(self,int ikeep):
    'expand the buffer if necessary, keeping elements [ikeep:nbuf]'
    cdef int length,istart
    cdef IntervalMap *new_buf
    istart=self.nbuf-ikeep
    length=sizeof(IntervalMap)*istart # #BYTES WE MUST KEEP
    if ikeep==0: # KEEPING THE WHOLE BUFFER, SO MUST ALLOCATE NEW SPACE
      new_buf=<IntervalMap *>realloc(self.im_buf,2*length) # DOUBLE OUR BUFFER
      if new_buf==NULL:
        raise MemoryError('out of memory')
      self.im_buf=new_buf
      self.nbuf=2*self.nbuf
    elif ikeep<8: # RUNNING OUT OF ROOM, SO EXPAND BUFFER
      self.nbuf=2*self.nbuf
      new_buf=interval_map_alloc(self.nbuf)
      memcpy(new_buf,self.im_buf+ikeep,length)
      free(self.im_buf)
      self.im_buf=new_buf
    else: # JUST SHIFT [ikeep:] SLICE OF BUFFER TO FRONT [0:istart]
      memmove(self.im_buf,self.im_buf+ikeep,length)
    return istart # RETURN START OF EMPTY BLOCK WHERE WE CAN ADD NEW DATA

  cdef int saveInterval(self,int start,int end,int target_id):
    'save an interval, expanding array if necessary'
    cdef int i
    if self.nhit>=self.nbuf: # EXPAND ARRAY IF NECESSARY
      self.extend(0)
    i=self.nhit
    self.im_buf[i].start=start # SAVE INTERVAL INFO
    self.im_buf[i].end=end
    self.im_buf[i].target_id=target_id
    self.nhit = i+1
    return self.nhit

  cdef int nextBlock(self,int *pkeep) except -2:
    'load one more block of overlapping intervals'
    cdef int i
    if self.it==NULL: # ITERATOR IS EXHAUSTED
      return -1
    if pkeep and pkeep[0]>=0 and pkeep[0]<self.nhit: #MUST KEEP [ikeep:] SLICE
      i=self.extend(pkeep[0]) # MOVE SLICE TO THE FRONT
    else: # WE CAN USE THE WHOLE BUFFER
      i=0
    if self.db is not None: # ON-DISK DATABASE
      find_file_intervals(self.it,self.start,self.end,
                          self.db.db[0].ii,self.db.db[0].nii,
                          self.db.db[0].subheader,self.db.db[0].nlists,
                          &(self.db.db[0].subheader_file),
                          self.db.db[0].ntop,self.db.db[0].div,
                          self.db.db[0].ifile_idb,
                          self.im_buf+i,self.nbuf-i,
                          &(self.nhit),&(self.it)) # GET NEXT BUFFER CHUNK
    elif self.idb is not None: # IN-MEMORY DATABASE
      find_intervals(self.it,self.start,self.end,self.idb.im,self.idb.ntop,
                     self.idb.subheader,self.idb.nlists,self.im_buf+i,self.nbuf-i,
                     &(self.nhit),&(self.it)) # GET NEXT BUFFER CHUNK
    else:
      raise IOError('Iterator has no database!  Please provide a db argument.')
    self.nhit=self.nhit+i # TOTAL #HITS IN THE BUFFER
    self.ihit=i # START ITERATING FROM START OF NEW HITS
    if pkeep and pkeep[0]>=0: # RESET ikeep INDEX TO START OF BUFFER
      pkeep[0]=0
    return self.nhit-self.ihit # RETURN #NEW HITS IN NEXT BLOCK

  cdef IntervalMap *getIntervalMap(self):
    '''return the IntervalMap array loaded by iterator,
    and release it from iterator.  User must free the array!'''
    cdef int len
    cdef IntervalMap *im
    if self.nhit==0: # NO HITS
      return NULL
    elif self.nhit<self.nbuf: # LARGER BUFFER THAN WE ACTUALLY NEED
      len=sizeof(IntervalMap)*self.nhit # COMPUTE FINAL SIZE
      im=<IntervalMap *>realloc(self.im_buf,len) # COMPACT TO FINAL SIZE
    else: # JUST HAND BACK OUR FULL BUFFER
      im=self.im_buf
    self.im_buf=NULL # RELEASE THIS STORAGE FROM ITERATOR; USER MUST FREE IT!
    return im # HAND BACK THE STORAGE

  cdef int loadAll(self) except -1:
    'load all overlapping interval hits, return count of hits'
    cdef int len,ikeep
    len=1
    ikeep=0 # DON'T LET extend DISCARD ANY HITS, KEEP THEM ALL!
    while len>0: # LOAD BLOCKS UNTIL NO MORE...
      len=self.nextBlock(&ikeep) # LOAD ANOTHER BLOCK OF INTERVALS
    return self.nhit

  cdef int cnext(self,int *pkeep): # C VERSION OF ITERATOR next METHOD
    'get one more overlapping interval'

    cdef int i
    if self.ihit>=self.nhit: # TRY TO GET ONE MORE BUFFER CHUNK OF HITS
      self.nextBlock(pkeep) # LOAD THE NEXT BLOCK IF ANY
    if self.ihit<self.nhit: # RETURN NEXT ITEM FROM BUFFER
      i=self.ihit
      self.ihit = self.ihit+1 # ADVANCE THE BUFFER COUNTER
      return i
    else: # BUFFER WAS EMPTY, NO HITS TO ITERATE OVER...
      return -1

  cdef int copy(self,IntervalFileDBIterator src):
    'copy items from src to this iterator buffer'
    cdef IntervalMap *new_buf
    if src is None:
      raise ValueError('src is None!  Debug!!')
    if src.nhit>self.nbuf: # NEED TO EXPAND OUR BUFFER
      new_buf=<IntervalMap *>realloc(self.im_buf,src.nhit*sizeof(IntervalMap))
      if new_buf==NULL:
        raise MemoryError('out of memory')
      self.im_buf=new_buf # RECORD NEW BUFFER LOCATION AND SIZE
      self.nbuf=src.nhit
    self.nhit=src.nhit # COPY ARRAY AND SET CORRECT SIZE
    if src.nhit>0: # ONLY COPY IF NON-EMPTY
      memcpy(self.im_buf,src.im_buf,src.nhit*sizeof(IntervalMap))
    return 0

  def __iter__(self):
    return self

  # PYTHON VERSION OF next RETURNS HIT AS A TUPLE
  def __next__(self): # PYREX USES THIS NON-STANDARD NAME INSTEAD OF next()!!!
    cdef int i
    i=self.cnext(NULL)
    if i>=0:
      return (self.im_buf[i].start,self.im_buf[i].end,self.im_buf[i].target_id )
    else:
      raise StopIteration

  def __dealloc__(self):
    'remember: dealloc cannot call other methods!'
    free_interval_iterator(self.it_alloc)
    if self.im_buf:
      free(self.im_buf)

      



